      program wts2pop

!  compile with 
!   gfortran -O3 -o wts2pop wts2pop-[date].f

!  if any problems with the code are suspected, use
!   gfortran -fbounds-check -g -O0 -o wts2pop wts2pop-[date].f
!      OR
!   gfortran -fbounds-check -gdwarf-2 -O0 -o wts2pop wts2pop-[date].f

!  [OLD] if your PATH is not correctly set up, try /usr/local/bin/g77

!  bin orientations from a ".WTS" file into a discrete 
!  S[ample] O[rientation] D[istribution], i.e. SOD in popLA format
!  which means slices in phi1
!  also COD, slices in phi2
!  also pole figures:
!    edit the list of generating poles in order to add different
!    pole figures

! version 1,  23 xii 01    adr
!  v2, iv 03  adr
!  v3, vi 03, adr - cleaned up sample symmetry so that only one
!  specification per component required
!  v4, xii 04, adr - added binning into triclinic sample symmetry
!  v5  ix 05, adr - added binning into inverse pole figures
!  v6  xi 05, adr - added SOD input
!  v7  ix 06, adr - longer file names
!  v8  xii 06, adr - command line arguments
!  v9  i 07, adr -  consolidate near-identical points on input (efficiency)
!  vii 07 - partial preparation for orthorhombic xtals
!  xii 07 - cleaned up some details in normalization of ODs (NORMOD)
!  ? 08 - inserted HCP capability
!  vi 08 - cleaned up ATAN2 problem for PF and IPF calc.
!  vii 08 - added CTF read capability (need to do ANG also!)
!  vii 08 - checked out HCP functionality
!  vii 08 - added output of discrete PF files (for GMT plotting)
!  vii 08 minor cleanup - no lines >72 (for John Butler)
!  viii 08 - added ability to read quaternion sets (Joel Bernier)
!  i  09  - cleaned up titles line for output of vol fracs
! iii 09  - allowed reading WTS file with arbitrary extension (e.g. TXT)
! iv 09   - starts sequence with 01, sorted arb. extension
! vii 09  - changed default output in q2eulB to give 0 instead of 45 degrees
!  ix 09  - corrected a problem in NORMOD with SODs out to 360 in phi1
!  x 09  - cleaned up the .comps file
!  iv 11 - hidden option to add 90 degr to phi-1
!  v  11  - minor corrections; capability to read GB segment files

!  DON"T FORGET TO UPDATE THE "LAST REVISED" print!!  

!  read orientations from a ".wts" file with Bunge or Kocks angles
!  symmetry elements from QUAT.SYMM.CUBIC
!  CSLs from QUAT.CSL

!  LOCAL VARIABLES

	implicit none

	real scalar
	integer i,j,k,l,ipoint,pc,dot_index
        integer indexf1,indexcf,indexf2,ijk,irange
	real theta,phi,d1,d2,d3,discomps
	logical result,repeat,only_one
	real qq(4),qq1(4),qend(4),quint(4),PI,twopi,degrad
	integer i9 , j9 , i9lower , i9upper , j9lower , j9upper

!  GLOBAL VARIABLES
	real qr1(4),qr2(4),qtwo(4,2),qresult(4),axis(3)
	integer ij,jk
!
	character*80 textitl
	character hkl*3,seclab*5,title*8,idate*8
!	integer numsymm,numcsl,nsig,ngrain,numsamp
!	real phi1,capphi,phi2

	integer intens(73,19,19),imax,iavg
	character prosa*80,prosa1*80,hh,kk,ll,lper,nomon
	character eulan*25,nomen,iper*6
	character*5 label,secid
	character fname*50,one,ten,fname2*50,tname*6
	real rintens(73,19,19),projs(73,19),cprojs(73,19),delph
	real triclinic(73,19,19)  !  for tricilinic sample symmetry SODs

	integer numpolfigs,numpoles(10)
	real hklgen(3,10)  !  allow for 10 pole figures
	real hklarray(3,10,48) ! all possible 48 general poles
	real tmp(3),tmp1(3),tmp2(3),rtmp(3)
	logical already
	real qqsym(4)
!	data numpolfigs /3/
!	data hklgen(1,1),hklgen(2,1),hklgen(3,1) /1.,0.,0./
!	data hklgen(1,2),hklgen(2,2),hklgen(3,2) /1.,1.,1./
!	data hklgen(1,3),hklgen(2,3),hklgen(3,3) /1.,0.,1./
	real polearray(73,19,10)  !  accumulator for pole fig
	integer ipolearray(73,19,10),iavgpf(10)  ! integer form of pole fig

	real invpfgen(3,10)  !  allow for 10 pole figures
	real ivparray(3,10,16)    !  store poles for IVPs
	real invpolearray(73,19,10)  !  accumulator for pole fig
	integer iinvpolearray(73,19,10),iavginvpf(10)  ! integer form of pole fig
	character type*3  !  pass to WPF
! }}}}  everything the same as for pole figures, just invert the rotations!

	integer iq , iq2
	logical read_another
	real rhkl(3),ruvw(3),dismin(20),wtcomp(20),tolerns,totwt,bbtmp
!  RHKL is for reading in HKL, RUVW for UVW for specifying components
!  DISMIN is used to track min. mis. angle for each component
!  WTCOMP is for the weighting assigned to each component
!  TOTWT is the total weight
	integer ncount  !  use for component count
	integer llimit,ulimit ! loop counters for names

	logical Kocks  ! if TRUE, then output in Kocks angles (for use with WEIGHTS)
	integer iq_sod  !  see below
	logical read_sod  !  if TRUE then input is from .SOD file(s)
	integer index_sod  !  for file name indexing with SOD
        logical l_NoWeight  !  for using uniform weight with each line (WTS only)
        integer NoWeight !  see above
        logical read_ctf  !  for reading CTF files (from Channel/HKL)
        logical read_wquat  !  for reading WQUAT (from Joel Bernier)
        logical read_any  !  for reading arb. extension WTS file
        logical read_GB   !  for reading GB segments files, angles in radians
        logical read_GB_degr   !  for reading GB segments files, angles in degrees

	integer numcsl,nsig(50),numsymm,numsamp,ngrain
	real quatsymm(4,48),quatsamp(4,4)
	real quatcsl(4,50)
	real aquat(4,2000000),phi1(2000000),capphi(2000000)
     $       ,phi2(2000000)
	real grwt(2000000)
	common/a1/ quatsymm,numsymm,quatsamp,numsamp
	common/a2/quatcsl,numcsl,nsig
	common/a3/aquat,ngrain,phi1,capphi,phi2
	common/a4/ grwt,eulan

        logical add_90
	common/strings/ fname,hkl,seclab,title,idate,textitl,add_90
	common /t1/ rintens,triclinic,projs,cprojs,intens,delph,iavg
!	common /t1/ rintens,projs,cprojs,intens,delph,iavg
	character*8 comp_name(20)	! names of texture components
	real compphi1(20,3),compcphi(20,3),compphi2(20,3)	!Euler angles
	real qcomp(20,3,4), qmis(4,2)		! quaternions for texture components
	integer comp_index(2000000),comp_index2(2000000)
	real thetamin , disorientmin , comp_vol(20) , sumvol , randpart
!  indexes to texture components
	integer numcomps
	integer time

        character vtitles*256
        integer vtitles_upper
        data vtitles_upper / 256 /
!  edited 9 Jan 09
	character*8 comp_name_xfer
	common /poles/ polearray,ipolearray,hklgen,iavgpf,numpolfigs

        character pfnum*1 ! use for writing discrete PFs
!        character secid*5,hh*1,kk*1,ll*1 ! also for discrete PFs
        integer ns

        integer num_args
        character inline*80

        integer date_time(8)
        character stamp*99
        character tim*8,dat*9

        integer iq_symm  !  crystal symmetry

        integer equivorth , equivtri  !  for counting how many copies in the space
        integer equivorthLim , equivtriLim
        parameter ( equivtriLim = 1 )       !  set limit for no. of copies, triclinic
        parameter ( equivorthLim = 1 )      !  set limit for no. of copies, orthorh.
        integer iqsampoff

        integer number2read  !  number of input lines to read
        integer itmp

!  CODE::
        call newdate(dat,tim)
        stamp = 'wts2pop: '//DAT//' '//TIM

        num_args = iargc()
!  arguments:
!   1   IQ
!   2   IQ_SOD  reading from WTS or SOD
!   3   filename base
!   4   weighting of points
!   5   symmetry

        if(num_args.le.0) then
           print*,'WTS2POP Usage:   Example:'
           print*,'/code/texture/wts2pop 0 0 generic.wts 0 cubic -1 15'
           print*
           print*,'In more detail ...'
           print*,'./wts2pop '
           print*,'1) Built-in_comps? [0=yes]'
           print*,'2) Read .WTS [ 0 ] or .SOD [ = 1 ] '
     $          ,'or .CTF [ = 2 ] or .WQUAT [ = 3 ]'
     $          ,'or .any-extension [ = 4 ] '
     $          ,'or a GB segment file [=5 for radians, 6 for degrees]'
           print*,'3)  filename '
           print*,'4)  read weights (for .WTS files) [0=yes, 1=no] '
           print*,'5)  crystal symmetry [ cubic | hex | orth ] '
           print*,'6)  number of lines to read [ -1 => no limit ] '
           print*,'7) capture radius, degrees [ try 15 ] '
           print *
        endif

!  are we using the components in the code?
        iq = -1                 !  use built-in components, or read in from STDIN
        if(num_args.gt.0) then
           CALL GetArg(1,inline)
!	print*,'debug: afile = ',afile
           if(inline.ne.'') then
              read(inline,"(i4)") iq
              if(iq.lt.0 .and. iq.gt.1) stop 'bad value for IQ'
           endif
        endif

!  normal is to read WTS files, but 1= read .SOD
        iq_sod = -1
        if(num_args.gt.1) then
           CALL GetArg(2,inline)
           read(inline,"(i4)") iq_sod
           if( iq_sod .eq. 0 ) print*,'reading WTS'
           if( iq_sod .eq. 1 ) print*,'reading SOD'
           if( iq_sod .eq. 2 ) print*,'reading CTF'
           if( iq_sod .eq. 3 ) print*,'reading WQUAT (Bernier)'
           if( iq_sod .eq. 4 ) print*,'reading WTS, arb extension (TXT)'
           if( iq_sod .eq. 5 ) print*,'reading GB seg. file, radians'
           if( iq_sod .eq. 6 ) print*,'reading GB seg. file, degrees'
           if( iq_sod.lt.0 .and. iq_sod.gt.6 ) then
              stop 'bad value for IQ_SOD'
           endif
        endif

!  now get the filename for the data file(s)
        fname2 = ''
        if(num_args.gt.2) then
           CALL GetArg(3,inline)
           ipoint = len_trim(inline)
           if( ipoint.gt.1 ) fname2 = inline(1:ipoint)
        endif

!  weights in col 4?
        NoWeight = -1
        if(num_args.gt.3) then
           CALL GetArg(4,inline)
!	print*,'debug: inline = ',inline
           l_NoWeight = .false.
           if(inline.eq.'1') then 
              l_NoWeight = .true.
              NoWeight = 1
           elseif(inline.eq.'0') then
!           l_NoWeight = .true.
              NoWeight = 0
           endif
        endif

        iq_symm = 0
        if(num_args.gt.4) then
           CALL GetArg(5,inline)
!           print*,'5th arg: ',inline
           ipoint = 0
           ipoint = index(inline,'cubic')
           if(ipoint.gt.0) then
              iq_symm = 1
              print*,'cubic xtal symmetry requested'
           endif
           if (ipoint .eq. 0 ) then
              ipoint = index(inline,'hex')
              if(ipoint.gt.0) then 
                 iq_symm = 2
                 print*,'hexagonal xtal symmetry requested'
              endif
           endif
           if (ipoint .eq. 0 ) then
              ipoint = index(inline,'orth')
              if(ipoint.gt.0) then 
                 iq_symm = 3
                 print*,'orthorhombic xtal symmetry requested'
              endif
           endif
           if(ipoint.eq.0) stop 'bad symmetry spec.?'
        else
           print*,'Which crystal symmetry ?'
           print*,'1=cubic 2=hex 3=orth ...'
           read*,iq_symm
           if( iq_symm .lt. 1 .or. iq_symm .gt. 3 ) then 
              iq_symm = 1
              print*,'cubic xtal symmetry assumed'
           endif
        endif

!        print*,'iq,NoWeight,iq_sod,fname2,iq_symm'
!        print*,iq,NoWeight,iq_sod,fname2,iq_symm

!  are we using the components in the code?
        number2Read = -1                 !  read as many lines as in the file
        if( num_args.gt.5 ) then
           CALL GetArg( 6 , inline )
           if(inline.ne.'') then
              read(inline,"(i4)") number2Read
              if( number2Read.lt.-1 ) stop 'bad value for number2Read'
           endif
        endif

	Kocks = .false.
	tolerns = 15.  !  just in case
        if( num_args.gt.6 ) then
           CALL GetArg( 7 , inline )
           if(inline.ne.'') then
              read( inline , * ) tolerns
              if( tolerns .le. 0. ) stop 'bad value for capture radius'
           end if
        end if
        print * , 'Capture angle for components = ' , tolerns

	add_90 = .false.
        if( num_args.gt.7 ) then
           CALL GetArg( 8 , inline )
           if(inline.ne.'') then
              read( inline , * ) itmp
              if( itmp .ne. 0 ) then
                 add_90 = .true.
                 print* ,'WILL ADD 90 DEGREES to PHI-1'
              end if
           end if
        end if

        if( iq_symm.eq.1 ) then
           numpolfigs=3
           hklgen(1,1)=1.
           hklgen(2,1)=0.
           hklgen(3,1)=0.
           hklgen(1,2)=1.
           hklgen(2,2)=1.
           hklgen(3,2)=1.
           hklgen(1,3)=1.
           hklgen(2,3)=0.
           hklgen(3,3)=1.

        elseif(iq_symm.eq.2) then
           numpolfigs=3
           hklgen(1,1) = 1.
           hklgen(2,1) = 0.
           hklgen(3,1) = 0.
!     gives (100) or (2 -1 -1 0) PF
           hklgen(1,2) = 0.
           hklgen(2,2) = 0.
           hklgen(3,2) = 1.
!     gives (001) or (0001) PF
           hklgen(1,3) = 0.
           hklgen(2,3) = 1.
           hklgen(3,3) = 0.
!     gives (1 0 -1 0) PF

        elseif( iq_symm.eq.3 ) then  !  orthorhombic
           numpolfigs=3
           hklgen(1,1) = 1.
           hklgen(2,1) = 0.
           hklgen(3,1) = 0.
!     gives (1 0 0) PF
           hklgen(1,2) = 0.
           hklgen(2,2) = 1.
           hklgen(3,2) = 0.
!     gives (0 1 0) PF
           hklgen(1,3) = 0.
           hklgen(2,3) = 0.
           hklgen(3,3) = 1.
!     gives (0 0 1) PF
        end if

	PI = 4. * atan ( 1. )
	degrad = 180 / PI
	twopi = 2.0 * PI
 8	format(a)

	call textur4(iq_symm)  !  read in the symmetry operators

	ncount=0
	print*,'List of built-in components:'

        if( iq_symm.eq.1 ) then
           ncount = ncount + 1
           comp_name(ncount) = '  cube  '
           compphi1(ncount,1) = 0.
           compcphi(ncount,1) = 0.
           compphi2(ncount,1) = 0.
c$$$           print*,'Comp ',ncount,': ',comp_name(ncount)
c$$$     &          ,compphi1(ncount,1),compcphi(ncount,1)
c$$$     $          ,compphi2(ncount,1)

! *****************  FCC  ***********************
      ncount = ncount + 1
      comp_name(ncount) = 'Goss__'
      compphi1(ncount,1) = 0.
      compcphi(ncount,1) = 45.
      compphi2(ncount,1) = 0.

      ncount = ncount + 1
      comp_name(ncount)='brass_'
      compphi1(ncount,1)=35.
      compcphi(ncount,1)=45.
      compphi2(ncount,1)=0.

c     S {123} <63-4>  Bunge
      ncount = ncount + 1
      comp_name(ncount)='123634'
      compphi1(ncount,1) = 58.98
      compcphi(ncount,1) = 36.70
      compphi2(ncount,1) = 63.43

      ncount = ncount + 1
      comp_name( ncount ) = 'G/Brss'
      compphi1(ncount,1) = 15.
      compcphi(ncount,1) = 45.
      compphi2(ncount , 1 ) = 0.

      ncount = ncount + 1
      comp_name(ncount)='S/Brss'
      compphi1(ncount,1)=42.
      compcphi(ncount,1)=37.
      compphi2(ncount,1)=80.

      ncount = ncount + 1
      comp_name(ncount)='Copper'
      compphi1(ncount,1) = 90.
      compcphi(ncount,1) = 35.
      compphi2(ncount,1) = 45.

      ncount = ncount + 1
      comp_name(ncount)='Dillam'
      compphi1(ncount,1)=42.
      compcphi(ncount,1)=71.
      compphi2(ncount,1)=20.

      ncount = ncount + 1
      comp_name(ncount)='123412'		! also known as "R" component
      compphi1(ncount,1)=68.76
      compcphi(ncount,1)=74.5
      compphi2(ncount,1)=56.31

      ncount = ncount + 1
      comp_name(ncount)='Sv2'
      compphi1(ncount,1)= 70
      compcphi(ncount,1)= 35.
      compphi2(ncount,1)= 30.

      ncount = ncount + 1
      comp_name(ncount)='nearBs'
      compphi1(ncount,1)= 30.
      compcphi(ncount,1)= 65.
      compphi2(ncount,1)= 45.

      ncount = ncount + 1
      comp_name(ncount)='rotCub'
      compphi1(ncount,1)= 30.
      compcphi(ncount,1)= 10.
      compphi2(ncount,1)= 20.
! *****************  FCC end  ***********************



! ============= BCC =================
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='001_110 ' !  45 ND rotated cube
c$$$           compphi1(ncount,1) = 0.
c$$$           compcphi(ncount,1) = 0.
c$$$           compphi2(ncount,1) = 45.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='110_011 '
c$$$           compphi1(ncount,1) = 45.
c$$$           compcphi(ncount,1) = 0.
c$$$           compphi2(ncount,1) = 45.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='55_65_25'
c$$$           compphi1(ncount,1) = 55.
c$$$           compcphi(ncount,1) = 65.
c$$$           compphi2(ncount,1) = 25.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='50_75_20'
c$$$           compphi1(ncount,1) = 50.
c$$$           compcphi(ncount,1) = 75.
c$$$           compphi2(ncount,1) = 20.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='45_80_15'
c$$$           compphi1(ncount,1) = 45.
c$$$           compcphi(ncount,1) = 80.
c$$$           compphi2(ncount,1) = 15.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='40_80_15'
c$$$           compphi1(ncount,1) = 40.
c$$$           compcphi(ncount,1) = 80.
c$$$           compphi2(ncount,1) = 15.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='90_00_45'
c$$$           compphi1(ncount,1) = 90.
c$$$           compcphi(ncount,1) = 0.
c$$$           compphi2(ncount,1) = 45.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='0_25_45'
c$$$           compphi1(ncount,1) = 0.
c$$$           compcphi(ncount,1) = 25.
c$$$           compphi2(ncount,1) = 45.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='15_20_55'
c$$$           compphi1(ncount,1) = 15.
c$$$           compcphi(ncount,1) = 20.
c$$$           compphi2(ncount,1) = 55.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='gamma__0'
c$$$           compphi1(ncount,1) = 0.
c$$$           compcphi(ncount,1) = 54.7
c$$$           compphi2(ncount,1) = 45.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='gamma_30'
c$$$           compphi1(ncount,1) = 30.
c$$$           compcphi(ncount,1) = 54.7
c$$$           compphi2(ncount,1) = 45.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='gamma_60'
c$$$           compphi1(ncount,1) = 60.
c$$$           compcphi(ncount,1) = 54.7
c$$$           compphi2(ncount,1) = 45.
c$$$
c$$$           ncount = ncount + 1
c$$$           comp_name(ncount)='gamma_90'
c$$$           compphi1(ncount,1) = 90.
c$$$           compcphi(ncount,1) = 54.7
c$$$           compphi2(ncount,1) = 45.

! ============= end of BCC =================

!$$$           ncount=ncount+1
!$$$           comp_name(ncount)='  NDcube'
!$$$           compphi1(ncount,1) = 45.
!$$$           compcphi(ncount,1) = 0.
!$$$           compphi2(ncount,1) = 0.
!$$$           print*,'Comp ',ncount,': ',comp_name(ncount)
!$$$     &          ,compphi1(ncount,1),compcphi(ncount,1)
!$$$     $          ,compphi2(ncount,1)

!     ncount=ncount+1
!     comp_name(ncount)='  10RDcb'
!     compphi1(ncount,1)=0.
!     compcphi(ncount,1)=10.
!     compphi2(ncount,1)=0.
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)

!     ncount=ncount+1
!     comp_name(ncount)='  20RDcb'
!     compphi1(ncount,1)=0.
!     compcphi(ncount,1)=20.
!     compphi2(ncount,1)=0.
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)

!     ncount=ncount+1
!     comp_name(ncount)='  30RDcb'
!     compphi1(ncount,1)=0.
!     compcphi(ncount,1)=30.
!     compphi2(ncount,1)=0.
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)

!     ncount=ncount+1
!     comp_name(ncount)='  rot_Gs'
!     compphi1(ncount,1)=90.0
!     compcphi(ncount,1)=45.0
!     compphi2(ncount,1)=0.0
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)

!$$$  ncount=ncount+1
!$$$  comp_name(ncount)='  20r321'
!$$$  compphi1(ncount,1)=216.388153
!$$$  compcphi(ncount,1)=19.26544
!$$$  compphi2(ncount,1)=329.008026
!$$$  print*,'Comp ',ncount,': ',comp_name(ncount)
!$$$  &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)

!     ncount=ncount+1
!     comp_name(ncount)='  30r321'
!     compphi1(ncount,1)=217.786163 
!     compcphi(ncount,1)= 28.8845215
!     compphi2(ncount,1)= 330.406036
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)

!     ncount=ncount+1
!     comp_name(ncount)='  20r123'
!     compphi1(ncount,1)=71.48
!     compcphi(ncount,1)=11.91
!     compphi2(ncount,1)=-55.39
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)
!     ncount=ncount+1
!     comp_name(ncount)='  30r123'
!     compphi1(ncount,1)=75.56
!     compcphi(ncount,1)=17.8
!     compphi2(ncount,1)=-51.31
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)
!     ncount=ncount+1
!     comp_name(ncount)='  10Copp'
!     compphi1(ncount,1)=85.86
!     compcphi(ncount,1)=29.27
!     compphi2(ncount,1)=130.80
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)
!     comp_name(ncount)='  20Copp'
!     compphi1(ncount,1)=81.81
!     compcphi(ncount,1)=23.14
!     compphi2(ncount,1)=126.74
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)

c$$$           ncount=ncount+1
c$$$           comp_name(ncount)='  Goss  '
c$$$           compphi1(ncount,1)=0.
c$$$           compcphi(ncount,1)=45.
c$$$           compphi2(ncount,1)=0.
c$$$           print*,'Comp ',ncount,': ',comp_name(ncount)
c$$$     &          ,compphi1(ncount,1),compcphi(ncount,1)
c$$$     $          ,compphi2(ncount,1)
c$$$
c$$$           ncount=ncount+1
c$$$           comp_name(ncount)='  Brass '
c$$$           compphi1(ncount,1)=35.
c$$$           compcphi(ncount,1)=45.
c$$$           compphi2(ncount,1)=0.
c$$$           print*,'Comp ',ncount,': '
c$$$     &          ,comp_name(ncount),compphi1(ncount,1)
c$$$     &          ,compcphi(ncount,1),compphi2(ncount,1)

!           ncount=ncount+1
!           comp_name(ncount)='  Dillam' !  4 4 11  11 11 -8
!           compphi1(ncount,1) = 45.
!           compcphi(ncount,1) = 0.
!           compphi2(ncount,1) = 30.
!           print*,'Comp ',ncount,': ',comp_name(ncount)
!     $          ,compphi1(ncount,1)
!     &          ,compcphi(ncount,1),compphi2(ncount,1)

!$$$!  Copper in Bunge angles
!$$$           ncount=ncount+1
!$$$           comp_name(ncount)='  Copper'
!$$$           compphi1(ncount,1) = 90.
!$$$           compcphi(ncount,1) = 35.
!$$$           compphi2(ncount,1) = 45.
!$$$           print*,'Comp ',ncount,': ',comp_name(ncount)
!$$$     &          ,compphi1(ncount,1),compcphi(ncount,1)
!$$$     $          ,compphi2(ncount,1)

!$$$           ncount=ncount+1
!$$$           comp_name(ncount)='  echCop'
!$$$           compphi1(ncount,1) = 90.
!$$$           compcphi(ncount,1) = 10.
!$$$           compphi2(ncount,1) = 45.
!$$$           print*,'Comp ',ncount,': ',comp_name(ncount)
!$$$     &          ,compphi1(ncount,1),compcphi(ncount,1)
!$$$     $          ,compphi2(ncount,1)

c$$$           ncount=ncount+1
c$$$           comp_name(ncount)=' 231-124' !  1 2 3 6 3 -4
c$$$           compphi1(ncount,1)=64.934
c$$$           compcphi(ncount,1)=74.499
c$$$           compphi2(ncount,1)=33.69
c$$$           print*,'Comp ',ncount,': ',comp_name(ncount)
c$$$     &          ,compphi1(ncount,1),compcphi(ncount,1)
c$$$     $          ,compphi2(ncount,1)
  
c$$$           ncount=ncount+1
c$$$           comp_name(ncount)=' 111-110' !  1 1 1 1 -1 0
c$$$           compphi1(ncount,1)=0.0
c$$$           compcphi(ncount,1)=54.7
c$$$           compphi2(ncount,1)=45.0
c$$$           print*,'Comp ',ncount,': ',comp_name(ncount)
c$$$     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)

!           ncount=ncount+1
!           comp_name(ncount)=' 110-110' !  1 1 0 1 -1 0
!           compphi1(ncount,1)=85.5
!     compcphi(ncount,1)=50.35
!     compphi2(ncount,1)=4.25
!     print*,'Comp ',ncount,': ',comp_name(ncount)
!     &  ,compphi1(ncount,1),compcphi(ncount,1),compphi2(ncount,1)

c$$$           ncount=ncount+1
c$$$           comp_name(ncount)=' 001-110' !  0 0 1 1 1 0
c$$$           compphi1(ncount,1)=0.
c$$$           compcphi(ncount,1)=0.
c$$$           compphi2(ncount,1)=45.

!     comp_name(8)=' 124-211'
!     compphi1(8,1)=9.63
!     compcphi(8,1)=77.4
!     compphi2(8,1)=63.431
!     compphi1(8,2)=86.59
!     compcphi(8,2)=64.12
!     compphi2(8,2)=75.96
!     compphi1(8,3)=56.79
!     compcphi(8,3)=29.21
!     compphi2(8,3)=26.57

!     following Euler angles are in Kocks angles! from hkl2eul
!     comp_name(6)=' 123-634'
!     compphi1(6,1)=80.83
!     compcphi(6,1)=74.5
!     compphi2(6,1)=56.31
!     compphi1(6,2)=0.93
!     compcphi(6,2)=57.69
!     compphi2(6,2)=18.43
!     compphi1(6,3)=31.02
!     compcphi(6,3)=36.7
!     compphi2(6,3)=26.57
!     comp_name(7)=' 123-412'		! also known as "R" component
!     compphi1(7,1)=68.76
!     compcphi(7,1)=74.5
!     compphi2(7,1)=56.31
!     compphi1(7,2)=11.14
!     compcphi(7,2)=57.69
!     compphi2(7,2)=71.57
!     compphi1(7,3)=43.09
!     compcphi(7,3)=36.7
!     compphi2(7,3)=26.57
!     comp_name(8)=' 124-211'
!     compphi1(8,1)=80.37
!     compcphi(8,1)=77.4
!     compphi2(8,1)=63.431
!     compphi1(8,2)=3.41
!     compcphi(8,2)=64.12
!     compphi2(8,2)=75.96
!     compphi1(8,3)=33.21
!     compcphi(8,3)=29.21
!     compphi2(8,3)=26.57

        elseif(iq_symm.eq.2) then
           ncount=ncount+1
           comp_name(1)='  TiCmp1'
           compphi1(ncount,1) = 0.
           compcphi(ncount,1) = 40.
           compphi2(ncount,1) = 0.
           print*,'Comp ',ncount,': ',comp_name(ncount)
     &          ,compphi1(ncount,1),compcphi(ncount,1)
     $          ,compphi2(ncount,1)
           ncount=ncount+1
           comp_name(ncount)='  TiCube'
           compphi1(ncount,1) = 0.
           compcphi(ncount,1) = 0.
           compphi2(ncount,1) = 0.
           print*,'Comp ',ncount,': ',comp_name(ncount)
     &          ,compphi1(ncount,1),compcphi(ncount,1)
     $          ,compphi2(ncount,1)

           ncount=ncount+1
           comp_name(ncount)='  Ti__90'
           compphi1(ncount,1) = 0.
           compcphi(ncount,1) = 90.
           compphi2(ncount,1) = 0.
           print*,'Comp ',ncount,': ',comp_name(ncount)
     &          ,compphi1(ncount,1),compcphi(ncount,1)
     $          ,compphi2(ncount,1)
           ncount=ncount+1
           comp_name(ncount)='  TiP_15'
           compphi1(ncount,1) = 15.
           compcphi(ncount,1) = 40.
           compphi2(ncount,1) = 60.
           print*,'Comp ',ncount,': ',comp_name(ncount)
     &          ,compphi1(ncount,1),compcphi(ncount,1)
     $          ,compphi2(ncount,1)

           ncount=ncount+1
           comp_name(ncount)='  TiP_30'
           compphi1(ncount,1) = 30.
           compcphi(ncount,1) = 40.
           compphi2(ncount,1) = 55.
           print*,'Comp ',ncount,': ',comp_name(ncount)
     &          ,compphi1(ncount,1),compcphi(ncount,1)
     $          ,compphi2(ncount,1)
           ncount=ncount+1
           comp_name(ncount)='  TiP_45'
           compphi1(ncount,1) = 45.
           compcphi(ncount,1) = 40.
           compphi2(ncount,1) = 45.
           print*,'Comp ',ncount,': ',comp_name(ncount)
     &          ,compphi1(ncount,1),compcphi(ncount,1)
     $          ,compphi2(ncount,1)

           ncount=ncount+1
           comp_name(ncount)='  Ti_Rex' !  Rex component
           compphi1(ncount,1) = 0.
           compcphi(ncount,1) = 30.
           compphi2(ncount,1) = 30.
           print*,'Comp ',ncount,': ',comp_name(ncount)
     &          ,compphi1(ncount,1),compcphi(ncount,1)
     $          ,compphi2(ncount,1)
        endif

	numcomps = ncount		!  number of texture components to check
        if ( numcomps .gt. 20 ) then
           print *,' edit code to have more than 20 components'
           numcomps = 20
        end if
	vtitles = '    Time(MCS)   '
	llimit = 15
	do i = 1 , numcomps
	   ulimit = llimit + 8  !  new for 31 x 09
	   comp_name_xfer = comp_name(i)
	   do j = 1 , 8
	      vtitles((j+llimit-1):(j+llimit-1)) = comp_name_xfer(j:j)
	   enddo
	   vtitles(ulimit:ulimit) = ' '
	   llimit = ulimit

	   comp_vol(i)=0.
	   do j=1,3
	      call quatB(compphi1(i,j)/degrad,compcphi(i,j)/degrad,
     1		        compphi2(i,j)/degrad,qr1)
	      do k=1,4
		 qcomp(i,j,k)=qr1(k)
	      enddo
	   enddo
	enddo

        do i = ulimit , vtitles_upper
           vtitles(ulimit:ulimit) = ' '
        end do
        vtitles_upper = len_trim ( vtitles )  !  new 31 x 09
        
	print*,'Title line = '
	print* , vtitles(1:vtitles_upper)

	do i = 1 , numpolfigs
!		write(*,*)
!		write(*,*) 'pole figure set number ',i
	   ipoint=1
	   tmp(1)=hklgen(1,i)
	   tmp(2)=hklgen(2,i)
	   tmp(3)=hklgen(3,i)
	   call donorm(tmp)
	   hklarray(1,i,1)=tmp(1)
	   hklarray(2,i,1)=tmp(2)
	   hklarray(3,i,1)=tmp(3)
!		write(*,"(i5,3(2x,f5.2))") ipoint,(tmp(j),j=1,3)
	   do j=1,numsymm
	      do k=1,4
		 qqsym(k)=quatsymm(k,j)
	      enddo

	      call qrvec(qqsym,tmp,tmp1)
!       write(*,*) 'symm oper no, tmp1: ',j,tmp1
!       apply a symmetry operator
	      already=.false.
	      do k=1,ipoint
		 tmp2(1)=hklarray(1,i,k)
		 tmp2(2)=hklarray(2,i,k)
		 tmp2(3)=hklarray(3,i,k)
		 if(abs(1.-scalar(tmp1,tmp2)).lt.1e-2) already=.true.
!	write(*,*) 'k,dot product,tmp1,tmp2 ',k,scalar(tmp1,tmp2),tmp1,tmp2
!       we've found a match for this pole
	      enddo

!       write(*,*) 'tmp1 ',tmp1
	      if(.not.already) then
		 ipoint=ipoint+1
		 do k=1,3
!       write(*,*) 'k,tmp1(k) ',k,tmp1(k)
		    hklarray(k,i,ipoint)=tmp1(k)
		      enddo
!       hklarray(2,i,ipoint)=tmp1(2)
!       hklarray(3,i,ipoint)=tmp1(3)
!       write(*,"('ipoint= ',i3,3(2x,f5.2))") 
!       &       ipoint,(hklarray(l,i,ipoint),l=1,3)
!       write(*,*) 'CHECK/symm oper no, tmp1: ',j,tmp1
		   endif
		enddo
		numpoles(i)=ipoint
	     enddo

! }}}}}
	     do i = 1,10
		invpfgen(1,i) = 0
		invpfgen(2,i) = 0
		invpfgen(3,i) = 0
		do j = 1,16   !  max number of eq poles for orthorhombic symmetry
		   ivparray(1,i,j) = 0.
		   ivparray(2,i,j) = 0.
		   ivparray(3,i,j) = 0.
		enddo
	     enddo

	     invpfgen(1,1) = 1
	     invpfgen(2,2) = 1
	     invpfgen(3,3) = 1
	     ivparray(1,1,1) = 1.
	     ivparray(1,1,2) = -1.
!  gives sample 100
	     ivparray(2,2,1) = 1.
	     ivparray(2,2,2) = -1.
!  gives sample 010
	     ivparray(3,3,1) = 1.
	     ivparray(3,3,2) = -1.
!  gives sample 001
	     
!  note for inv pole figures, for now: since we are going to calculate
!  only the RD, TD, ND (sample 1, 2 & 3) directions, there's no need 
!  for this apparatus, other than the set of individual points.

	write(*,*)
	write(*,*) 'WTS2POP is primarily for binning lists of discrete '
	write(*,*) 'orientations into discretized orientation '
	write(*,*) 'distributions in popLA format for subsequent'
	write(*,*) 'analysis or plotting.'
	print*
	write(*,*) 'It also performs a volume fraction analysis'
	print*,' Look for the volume fraction output in a *.comps file'
        print*,' also outputs standard popLA .COD, .SOD'
        print*,' also pole figures, and inverse PFs as .ppf, .ipf'
        print*,' also the first 10000 data points as .dpf and dip'
	print*
	print*,'Sample symmetry is treated as follows:'
	print*,'Each orientation is binned assuming triclinic (none)'
	print*,'  symmetry, and also assuming orthorhombic (mmm)'
	print*,'The output for triclinic has _360 added to the name'
	print*,'whereas the orthorhombic just has the rootname'
	write(*,*)
	print*,'  Written by A.D. Rollett, Carnegie Mellon University'
	print*,'  Please acknowledge use of the program in whatever'
	print*,'  fashion seems appropriate and inform me of errors'
	print*,'  at    rollett@andrew.cmu.edu '
	print*
	print*,'     last revised : 9 May 2011'
        print*
        print*,'Note that the capture/tolerance angle = '
     $       , tolerns , ' degrees'
        print*,'To compare against random, use random200k.wts as input'
        print*
        if(iq_symm.eq.2) then
           print*
           print*,' IMPORTANT: HKL in the pole figures is w.r.t.'
           print*,' the Cartesian reference axes: '
           print*,' There are two possible settings:'
           print*,'A: in Metz, e.g., with Bunge Euler angles, '
           print*,' the X-axis is parallel to 1 0 -1 0 , NOT 2 -1 -1 0'
           print*,' Thus, (100) represents (1 0 -1 0)'
           print*,' Thus, (010) represents (2 -1 -1 0)'
           print*
           print*,'B: in CMU, e.g., with Bunge Euler angles, '
           print*,' the X-axis is parallel to  2 -1 -1 0, NOT 1 0 -1 0'
           print*,' Thus, (100) represents (2 -1 -1 0)'
           print*,' Thus, (010) represents (1 0 -1 0)'
           print*
        endif
        if(iq.eq.-1) then
           print*,'Use the built-in set of components'
           print*,' (see the source code for this) [=0]'
           print*,'or enter the individual components by hand [=1]?'
           read(*,*) iq
        endif
	if(iq.eq.0) then
	   continue		! nothing new to do
	elseif(iq.eq.1) then
	   write(*,*) 'Do you want to enter Miller Indices [=0]'
	   write(*,*) 'or, Euler angles (Bunge!) [=1]'
	   read(*,*) iq2
	   pc=1
	   read_another=.true.
	   if(iq2.eq.0) then
	      write(*,*) 'Keep entering h k l u v w (max 12)'
	      write(*,*) 'When you are done, enter 999. 0 0 0 0 0'
	      do jk=1,12
		 numcomps=pc	!  just to be sure
		 if(read_another) then
		    write(*,*) 'Enter indices for '
     $                   ,jk,'th comp. (avoid h=k=0)'
		    read(*,*) rhkl,ruvw
		    if( rhkl(1).eq.999. ) then
		       read_another=.false.
		       numcomps=pc-1
		    else
		       write(*,*) 'and 6-char name for this component'
		       read(*,"(a)") comp_name(pc)
		       call donorm(rhkl)
		       call donorm(ruvw)
		       bbtmp = rhkl(3) / (sqrt(rhkl(1)**2
     $                      + rhkl(2)**2+rhkl(3)**2))
		       if(bbtmp.gt.1.0) bbtmp=1.0
		       if(bbtmp.lt.-1.0) bbtmp=-1.0
		       compcphi(pc,1)=degrad*acos(bbtmp)
		       bbtmp=rhkl(2)/(sqrt(rhkl(1)**2+rhkl(2)**2))
		       if(bbtmp.gt.1.0) bbtmp=1.0
		       if(bbtmp.lt.-1.0) bbtmp=-1.0
		       compphi2(pc,1)=degrad*acos(bbtmp)
		       bbtmp = ruvw(3) * sqrt((rhkl(1)**2
     $                      +rhkl(2)**2+rhkl(3)**2)
     &                      / (rhkl(1)**2+rhkl(2)**2)
     $                      / (ruvw(1)**2+ruvw(2)**2+ruvw(3)**2))
		       if(bbtmp.gt.1.0) bbtmp=1.0
		       if(bbtmp.lt.-1.0) bbtmp=-1.0
		       compphi1(pc,1)=degrad*acos(bbtmp)
		       write(*,*) 'check the Euler Angles: ',
     &		    compphi1(pc,1),compcphi(pc,1),compphi2(pc,1)
		       pc=pc+1
		    endif
!       number of texture components to check
		 endif
	      enddo
	   elseif(iq2.eq.1) then
	      write(*,*) 'Keep entering phi1 PHI phi2 (max 12)'
	      write(*,*) 'When you are done, enter 999. 999. 999.'
	      do jk=1,12
		 numcomps=pc	!  just to be sure
		 if(read_another) then
		    write(*,*) 'Enter angles for ',jk,'th comp.'
		    read(*,*) compphi1(pc,1),compcphi(pc,1)
     $                   ,compphi2(pc,1)
		    write(*,*) 'and 6-char name for this component'
		    read(*,"(a)") comp_name(pc)
		    if(compphi1(pc,1).eq.999.) then
		       read_another=.false.
		       numcomps=pc-1
		    endif
!       number of texture components to check
		    pc=pc+1
		 endif
	      enddo
	   else
	      stop 'not an option!'
	   endif
	else
	   stop 'not an option!'
	endif

	iq2 = 0
!$$$	write(*,*)
!$$$	print*,' Output in Bunge [default] or Kocks angles [=1]?'
!$$$	read(*,*) iq2
!$$$	if(iq2.eq.1) then
!$$$	   Kocks = .true.
!$$$	   print*,'Kocks angles will be used for output!'
!$$$	endif
!  so rarely need the Kocks angle output that I'm discarding it, xi 05

        if(iq_sod.lt.0) then
           print*
           print*,'Read .wts [ = 0 ], or .sod [ = 1 ]'
     $          ,', or .CTF [ = 2 ] , or .WQUAT [ = 3 ], '
     $          ,'or .any-extension [ = 4 ],'
     $          ,'or GB seg. file [ = 5 for radians, 6 for degrs. ] ?'
           read(*,*) iq_sod
        endif
	if( iq_sod .lt. 0 .or. iq_sod .gt. 6 ) stop 'illegal entry'
	read_sod = .false.
	if( iq_sod .eq. 1 ) then
           read_sod = .true.
           print*,'The output files will have _Z at the end of the root'
           print*,' to differentiate them from the input set'
        end if
	read_ctf = .false.
	if( iq_sod .eq. 2 ) then
           read_ctf = .true.
        end if
	read_wquat = .false.
	if( iq_sod .eq. 3 ) then
           read_wquat = .true.
        end if
	read_any = .false.
	if( iq_sod .eq. 4 ) then
           read_any = .true.
        end if
	read_GB = .false.
	if( iq_sod .eq. 5 ) then
           read_GB = .true.
        end if
	read_GB_degr = .false.
	if( iq_sod .eq. 6 ) then
           read_GB_degr = .true.
        end if

	print*,'__________'
        if(fname2.eq.'') then
           print*,'Enter the KEYWORD for this run, e.g. cube55'
           print*, 'Operates on a series of .wts files from a gr gr/rex'
           print*, 'simulation with names like cube55.11.wts (or .sod),'
           print*, ' cube55.21.wts, etc.'
           print*, 'If there is only one file with the name cube55.wts,'
           print*, '  wts2pop will try to analyze that one first.'
           only_one=.false.
           read(*,"(a)") fname2
        endif
!$$$	pointx=20
!$$$	do 5, i=20,1,-1
!$$$c		print*, 'i,fname2(i:i) ',i,fname2(i:i)
!$$$	   if(fname2(i:i).eq.' ') pointx=i
!$$$ 5	enddo
        dot_index = 0
	dot_index = index(fname2,'.')
        if(dot_index.eq.0) dot_index = index(fname2,' ')
        dot_index = dot_index - 1
!	fname = fname2(1:dot_index)
	fname = fname2(1:dot_index)//'_Z'
!  the extra 2 characters will be used for output when using SOD as input
	index_sod = dot_index
	if(read_sod) index_sod = index_sod+2
!  see above for filename modification for writing out results from SOD input

!	fname2((dot_index+1):(dot_index+1)) = char(0)
!	print*, 'fname2,dot_index,fname'
!	print*, fname2,',   ',dot_index,',   ',fname

	open(unit=2,file=fname(1:index_sod)//'.comps',status='unknown')
        write(2,*) vtitles(1:vtitles_upper)

!  do we need to enquire about weighting?
        if(NoWeight.lt.0) then
           l_NoWeight = .false.
           if(.not.read_sod) then
              print*,'weights from .WTS input [=0], or unweighted [=1]?'
              read(*,*) NoWeight
              if(NoWeight.eq.1) then
                 l_NoWeight = .true.
              else
                 NoWeight = 0
              endif
           endif
        endif

!  now deal with crystal symmetry if not specified in arguments
        if ( iq_symm .eq. 0 ) then
           print*,'Enter xtal symmetry [cubic/hex/orth] :'
           read"(a)",inline
           ipoint = 0
           ipoint = index(inline,'cubic')
           if(ipoint.gt.0) then
              iq_symm = 1
              print*,'cubic xtal symmetry requested'
           endif
           if (ipoint .eq. 0 ) then
              ipoint = index(inline,'hex')
              if(ipoint.gt.0) then 
                 iq_symm = 2
                 print*,'hexagonal xtal symmetry requested'
              endif
           endif
           if (ipoint .eq. 0 ) then
              ipoint = index(inline,'orth')
              if(ipoint.gt.0) then 
                 iq_symm = 3
                 print*,'orthorhombic xtal symmetry requested'
              endif
           endif
           if(ipoint.eq.0) stop 'bad symmetry spec.?'
        endif

!  check to see what's out there ...
	if( read_sod ) then
	   inquire(file=fname2(1:(dot_index))//'.sod',exist=only_one)
	endif
        if ( iq_sod .eq. 0 ) then
	   inquire(file=fname2(1:(dot_index))//'.wts',exist=only_one)
	endif
        if ( iq_sod .eq. 4 ) then
	   inquire( file=fname2 , exist=only_one )
	endif
	if( read_ctf ) then
	   inquire(file=fname2(1:(dot_index))//'.ctf',exist=only_one)
        endif
	if( read_wquat ) then
	   inquire(file=fname2(1:(dot_index))//'.wquat',exist=only_one)
        endif
	if( read_GB .or. read_GB_degr ) then
	   inquire(file=fname2(1:(dot_index))//'.txt',exist=only_one)
        endif

	if(only_one) then
	   time = 0
	   if( iq_sod .eq. 5 .or. iq_sod .eq. 6 ) then
	      print*, 'found ',fname2(1:(dot_index))//'.txt'
	   endif
	   if( iq_sod .eq. 4 ) then
	      print*, 'found ' , fname2
	   endif
	   if( iq_sod .eq. 3 ) then
	      print*, 'found ',fname2(1:(dot_index))//'.wquat'
	   endif
	   if( iq_sod .eq. 2 ) then
	      print*, 'found ',fname2(1:(dot_index))//'.ctf'
	   endif
	   if( iq_sod .eq. 1 ) then
	      print*, 'found ',fname2(1:(dot_index))//'.sod'
	   endif
	   if( iq_sod .eq. 0 ) then
	      print*, 'found ',fname2(1:(dot_index))//'.wts'
	   endif
	else
	   print*, 'did not find: '
     $          , fname2(1:(dot_index))//'.wts/.sod/.ctf/arb'
	   print*, 'Will look for a series of WTS/SOD files'
	endif

	i9upper = 9
	i9lower = 0
	if(only_one) then
           i9upper = 1
           i9lower = 1
        end if
	do 9000, i9 = i9lower , i9upper
	   one = char(48+i9)

!       loop for first time counter: change if different time stepping used!
	   j9upper = 9
	   j9lower = 0
	   if(only_one) then
              j9upper=1
              j9lower=1
           end if
           do 8990, j9 = j9lower , j9upper
	      do i=1,73
		 do j=1,19
		    do k=1,10
		       polearray(i,j,k)=0.
		       ipolearray(i,j,k)=0
		       invpolearray(i,j,k)=0.
		       iinvpolearray(i,j,k)=0
		    end do
		 end do
	      end do
!      zero out the pole & inv. pole figure accumulators

	      do 10, i=1,72
		 do j=1,19
		    do k=1,19
		       rintens(i,j,k)=0.0
		       intens(i,j,k)=0
		       triclinic(i,j,k) = 0.
		    end do
		 end do
 10	      end do
!       zero out the OD accumulators

	      if(.not.only_one) then
		 time=int(float(j9)*10**i9)
		 ten = char(48+j9)
		 fname = fname2(1:dot_index)//'.'//ten//one
                 print*,'debug: fname= ',fname
		 if( read_GB .or. read_GB_degr ) then
		    inquire(file=fname(1:(dot_index+3))//'.txt'
     $                   ,exist=repeat)
		 end if
		 if( read_wquat ) then
		    inquire(file=fname(1:(dot_index+3))//'.wquat'
     $                   ,exist=repeat)
		 endif
		 if(read_ctf) then
		    inquire(file=fname(1:(dot_index+3))//'.ctf'
     $                   ,exist=repeat)
		 endif
		 if(read_sod) then
		    inquire(file=fname(1:(dot_index+3))//'.sod'
     $                   ,exist=repeat)
		 endif
                 if ( iq_sod .eq. 0 ) then
		    inquire(file=fname(1:(dot_index+3))//'.wts'
     $                   ,exist=repeat)
		 endif
		 if(.not.repeat) then
		    print*, 'Did not find '
     $                   ,fname(1:(dot_index+3))//'.wts/sod'
		    goto 8990
		 endif
		 print*, 'Found ',fname(1:(dot_index+3))//'.wts/sod/ctf/txt'
	      endif
!  figure out file names

	      do 6,i=1,100000
		 phi1(i)=0.
		 capphi(i)=0.
		 phi2(i)=0.
		 grwt(i)=0.
		 aquat(1,i)=0.
		 aquat(2,i)=0.
		 aquat(3,i)=0.
		 aquat(4,i)=0.
		 comp_index(i)=0
		 comp_index2(i)=0
 6	      continue
	      do 7, i = 1 , numcomps
		 comp_vol(i) = 0.
 7	      end do

	      if ( read_GB .or. read_GB_degr ) then
                 call texture_GB( dot_index , only_one
     $                , l_NoWeight , number2Read , read_GB
     $                , read_GB_degr )
              end if

	      if ( read_any ) then
                 call texture_any( fname2 , only_one , l_NoWeight
     $                , number2Read )
              end if

	      if ( read_wquat ) then
                 call texture_wquat( dot_index
     $                , only_one , l_NoWeight , number2Read )
              end if

	      if ( read_ctf ) then
                 call texture_ctf(dot_index,only_one,l_NoWeight
     $                , number2Read )
              end if

	      if ( read_sod ) then
		 call readsod( dot_index , only_one ) !  get the texture data
	      end if
              if ( iq_sod .eq. 0 ) then
		 call textur5(dot_index,only_one,l_NoWeight , number2Read )
	      end if
	      print*, 'number of grains read in = ',ngrain

!       get the orientations, symms etc.

!  open files for discrete pole figures
              do j = 1 , numpolfigs
                 write(pfnum,"(i1.1)") j
                 open(unit = j+10
     $                , file = fname(1:index_sod)//"_"//pfnum//'.dpf'
     $                , status = 'unknown' )
                 ns=hklgen(1,j)
                 hh=char(48+ns)
                 ns=hklgen(2,j)
                 kk=char(48+ns)
                 ns=hklgen(3,j)
                 ll=char(48+ns)
                 secid='('//hh//kk//ll//')'
                 write( j+10 , "(5a)" ) secid , '  '
     $                , stamp(1:20) , ' / ' , textitl(1:66)
              enddo  !  num. PFs
              do j = 1 , 3
                 write(pfnum,"(i1.1)") j
                 open(unit = j+20
     $                , file = fname(1:index_sod)//"_"//pfnum//'.dip'
     $                , status = 'unknown' )
                 ns = invpfgen(1,j)
                 hh = char(48+ns)
                 ns = invpfgen(2,j)
                 kk = char(48+ns)
                 ns = invpfgen(3,j)
                 ll = char(48+ns)
                 secid = '('//hh//kk//ll//')'
                 write( j+20 , "(5a)" ) secid , '  '
     $                , stamp(1:20) , ' / ' , textitl(1:66)
              enddo  !  num. IPFs
!  start calcs.
	      sumvol = 0.		! use for total volume

!              print*,'turn off sample symmetry? yes = 1'
!              read*,iqsampoff
!              if( iqsampoff .eq. 1 ) numsamp = 1
!  leaving these lines in for now, but, part of the attempt to
!  be able to read in weak textures correctly

	      do 1000, i = 1 , ngrain
                 equivorth = 0
                 equivtri = 0  !  counting no of copies
		 if(mod(i,10000).eq.0) then
		    print("('Working on grain no. ',i9)"),i
		 endif
		 sumvol = sumvol + grwt(i)
		 qq1(1)=aquat(1,i)
		 qq1(2)=aquat(2,i)
		 qq1(3)=aquat(3,i)
		 qq1(4)=aquat(4,i)
		 qmis(1,1)=aquat(1,i)
		 qmis(2,1)=aquat(2,i)
		 qmis(3,1)=aquat(3,i)
		 qmis(4,1)=aquat(4,i)
  
!       now to calculate each pole figure
    
		 do j = 1 , numpolfigs
!       print*, 'pole figure no. ',j
		    do k = 1 , numpoles(j)
		       do l=1,3
			  tmp(l)=hklarray(l,j,k)
		       enddo
!                       print*, 'pole no. ',k,'; dirn= ',tmp
		       call qrvec ( qq1 , tmp , tmp1 )
!                       call invqrvec( qq1 , tmp , tmp1 )
!                       print*,'k: ',k,' rotated vec: ',tmp1
!       rotate each pole by the grain's orientation
		       call donorm(tmp1) !  just to be sure
		       if( tmp1(3) .ge. 0. ) then 
! don't index if on lower hemisph.
			  if( tmp1(3) .gt. 1.0 ) tmp1(3) = 1.0 !  just in case
			  theta = acos(tmp1(3))*degrad ! declination
       
			  if((abs(tmp1(2)).lt.1e-35)
     $                         .and. (abs(tmp1(1)).lt.1e-35)) then
			     phi = pi/4.
			  else
			     phi = atan2(tmp1(2),tmp1(1))*degrad ! azimuth
!  changed the order of 1 and 2, because in fortran it's atan2(y-component, x-component)
!   see L3_Components_pt2_16Jan08_v6.ppt  for notes;  ADR, jun 08
			  endif
			  if(phi.lt.0.) phi = phi+360.
!  write out to discrete PFs - no more than 10000 points
                          if ( i .lt. 10000 ) then
                             write ( j+10 , "(3(1x,f8.3))")
     $                            phi , 90.-theta , grwt(i)
                          endif

!  now we calc indexes
			  indexf1 = int(theta/5.0+0.5)+1
			  if(indexf1.lt.1.or.indexf1.gt.19) then
			     print*, 'problem: indexf1, theta = '
     $                            ,indexf1,theta
			     stop
			  endif
			  indexcf = int(phi/5.0+0.5)+1
			  if(indexcf.lt.1.or.indexcf.gt.73) then
			     print*, 'problem: indexcf, phi = '
     $                            ,indexcf,phi
			     stop
			  endif
!	print*, 'th,ph,i1,i2 ',theta,phi,indexf1,indexcf
                          polearray(indexcf,indexf1,j)
     $                         = polearray(indexcf,indexf1,j)+grwt(i)
		       endif
		    enddo	! loop over k, the set of poles
		 enddo		! loop over j, each PF
!       END OF POLE FIGURE CALC

! }}}}  BEGIN INVERSE POLE FIGURE - RD, TD and ND
		 do j=1,3
!       print*, 'inverse pole figure no. ',j
		    do k=1,2
!  always 2 poles for a sample direction, + and -
		       do l=1,3
			  tmp(l) = ivparray(l,j,k)
		       enddo
!       print*, 'pole no. ',k,'; dirn= ',tmp
                       do  ijk = 1,numsymm ! crystal symmetry
                          call postsymm(qq1,ijk,quint)
                          call invqrvec(quint,tmp,tmp1)
!       rotate each inverse pole by the grain's orientation back to crystal axes
                          call donorm(tmp1) !  just to be sure
                          if(tmp1(3).ge.0.) then ! don't index if on lower hemisph.
                             if(tmp1(3).gt.1.0) tmp1(3)=1.0 !  just in case
                             theta=acos(tmp1(3))*degrad ! declination
       
                             if((abs(tmp1(2)).lt.1e-35).and.
     1                            (abs(tmp1(1)).lt.1e-35)) then
                                phi=pi/4.
                             else
                                phi=atan2(tmp1(2),tmp1(1))*degrad ! azimuth
!  changed the order of args 1 and 2 - see above
                             endif
                             if(phi.lt.0.) phi=phi+360.
!  write out to discrete PFs - no more than 500 points
                             if ( i .lt. 500 ) then
                                write ( j+20 , "(3(1x,f8.3))" )
     $                               phi , 90.-theta , grwt(i)
                             endif

!  now we calc indexes
                             indexf1=int(theta/5.0+0.5)+1
                             if(indexf1.lt.1.or.indexf1.gt.19) then
                                print*, 'problem: indexf1, theta = '
     $                               ,indexf1,theta
                                stop
                             endif
                             indexcf=int(phi/5.0+0.5)+1
                             if(indexcf.lt.1.or.indexcf.gt.73) then
                                print*, 'problem: indexcf, phi = '
     $                               ,indexcf,phi
                                stop
                             endif
!	print*, 'th,ph,i1,i2 ',theta,phi,indexf1,indexcf
                             invpolearray(indexcf,indexf1,j) = 
     1                            invpolearray(indexcf,indexf1,j)
     $                            + grwt(i)
                          endif
                       enddo
		    enddo	! loop over the set of IPFs
		 enddo		! loop over each IPF
!       END OF INVERSE POLE FIGURE CALCULATION

!       now we run through all the identified texture components
!       and identify the closest one (minimum misorientation)
       
		 disorientmin=15.
		 do j=1,numcomps
		    dismin(j)=999999.
		    wtcomp(j)=0.
		 enddo
       
!       ###  the value of DISORIENTMIN sets the criterion for how close
!       an orientation must be to be identified as a special component
       
!       do 300, j=1,numcomps
!       do 280, k=1,3	! variants of each component
!       qmis(1,2)=qcomp(j,k,1)
!       qmis(2,2)=qcomp(j,k,2)
!       qmis(3,2)=qcomp(j,k,3)
!       qmis(4,2)=qcomp(j,k,4)
!       call misquat(qmis,thetamin)
!       if(thetamin.lt.disorientmin) then
!       disorientmin=thetamin
!       comp_index(i)=j
!       comp_index2(i)=k
!       endif
!       280			continue
!       300		continue
!       if(comp_index(i).ne.0) then
!       comp_vol(comp_index(i))=comp_vol(comp_index(i))+grwt(i)
!       print*, 'Component identified for grain number ',i
!       print*, 'Index= ',comp_index(i),
!	1		     '   Variant Index= ',comp_index2(i)
!       print*, 'Component name = ',comp_name(comp_index(i))
!       endif
       
		 do 900, j=1,numsymm ! crystal symmetry
		    call postsymm(qq1,j,quint)

!**** this next section bins the result with ONLY crystal symm applied       
		    call q2eulB(d1,d2,d3,quint)
		    if(d1.lt.0.0) d1=d1+twopi
		    if(d2.lt.0.0) d2=d2+twopi
		    if(d3.lt.0.0) d3=d3+twopi
		    d1=d1*degrad
		    d2=d2*degrad
		    d3=d3*degrad
!     convert to Bunge Euler angles
!     print*, d1, d2, d3
		    if(Kocks) then
		       d1 = d1 - 90.
		       if(d1.lt.0.) d1 = d1 + 360.
		       d3 = 90. - d3
		       if(d3.lt.0.) d3 = d3 + 360.
		    endif	!  convert to Kocks

                    if(iq_symm.eq.1) then
                       call testangs_tri(d1,d2,d3,result)
                    elseif(iq_symm.eq.2) then
                       call testangs_tri_hex(d1,d2,d3,result)
                    elseif(iq_symm.eq.3) then
                       call testangs_tri_ort(d1,d2,d3,result)
                    endif
		    if( result .and. equivtri.lt.equivtriLim ) then
!     this should yield 1 point per orientation
!     thereby enforcing the cubic crystal symmetry within the 360x90x90 space

                       equivtri = equivtri + 1

		       indexf1 = int(d1/5.0+0.5)+1
		       indexcf = int(d2/5.0+0.5)+1
		       indexf2 = int(d3/5.0+0.5)+1
!     DEBUG LINES ->
!$$$                       print*
!$$$                       print*, 'At triclinic section'
!$$$                       print*, 'Angles for grain number ',i
!$$$                       print*, phi1(i),capphi(i),phi2(i)
!$$$                       print*, 'Angles returned from sort '
!$$$                       print*, d1, d2, d3
!$$$                       print*, 'Loop Indices i , j = ',i,j
!$$$                       print*, 'Binning for this grain:'
!$$$                       print*, indexf1,indexcf,indexf2
     
		       triclinic(indexf1,indexcf,indexf2)=
     1                      triclinic(indexf1,indexcf,indexf2)+grwt(i)
		    endif
!**** END triclinic section

		    do 880, k=1,numsamp ! sample symmetry
		       call presamp(quint,k,qend)
!       now we try each component and see if we find a smaller misorientation
		       do ijk=1,numcomps
			  do jk=1,4
			     qmis(jk,1)=qend(jk) ! gets the current orient
			     qmis(jk,2)=qcomp(ijk,1,jk) ! gets the component
			  enddo !  

                          if(iq_symm.eq.1) then
                             call misquat(qmis,thetamin)
                          elseif(iq_symm.eq.2) then
                             call comquat(qmis,qr1)
                             call disquat_ang_hex(qr1,thetamin)
!  this should speed up the calculation (uses one only copy of xtal symmetry)
                          elseif(iq_symm.eq.3) then
                             call comquat(qmis,qr1)
                             call disquat_orth(qr1,thetamin)
!  this should speed up the calculation (uses one only copy of xtal symmetry)
                          endif
!  new for v10 with hex
			  if(thetamin.lt.dismin(ijk)) then
			     dismin(ijk)=thetamin
			  endif
		       enddo	! ijk loop on components

		       call q2eulB(d1,d2,d3,qend)
		       if(d1.lt.0.0) d1=d1+twopi
		       if(d2.lt.0.0) d2=d2+twopi
		       if(d3.lt.0.0) d3=d3+twopi
		       d1 = d1*degrad
		       d2 = d2*degrad
		       d3 = d3*degrad
!       convert to Bunge Euler angles
!       print*, d1, d2, d3
		       if ( Kocks ) then  ! but this is disabled
			  d1 = d1 - 90.
			  if(d1.lt.0.) d1 = d1 + 360.
			  d3 = 90. - d3
			  if(d3.lt.0.) d3 = d3 + 360.
		       end if	!  convert to Kocks

                       if(iq_symm.eq.1) then
                          call testangs(d1,d2,d3,result)
                       elseif(iq_symm.eq.2) then
                          call testangs_hex(d1,d2,d3,result)
                       elseif(iq_symm.eq.3) then
                          call testangs_ort(d1,d2,d3,result)
                       endif
!       DEBUG LINES ->
                       if ( i .eq. -2 ) then
                          print*,
                          print*, 'debug: number, angles, RESULT ',i
                          print*, phi1(i),capphi(i),phi2(i),result
                          if ( d1.lt.90. .and. d2.lt.90.
     $                         .and. d3.lt.90.) then
                             print*, 'debug: Angles from sort '
                             print*, d1, d2, d3,' all < 90'
                          else
                             print*, 'debug: Angles from sort '
                             print*, d1, d2, d3
                          end if
                          print*, 'debug: xtal , sample = ' , j , k	
			  indexf1 = int( d1 / 5.0 + 0.5 ) + 1
			  indexcf = int( d2 / 5.0 + 0.5 ) + 1
			  indexf2 = int( d3 / 5.0 + 0.5 ) + 1
                          print*, 'debug: Binning for this grain:'
                          print*, indexf1,indexcf,indexf2
                       end if

!  new in v10 to combine hex with cubic
		       if( result .and. equivorth.lt.equivorthLim ) then
!       this should yield 3 points per orientation
!       thereby enforcing the cubic crystal + ortho sample symmetry in the 90x90x90 space

                          equivorth = equivorth + 1
			  indexf1 = int( d1 / 5.0 + 0.5 ) + 1
			  indexcf = int( d2 / 5.0 + 0.5 ) + 1
			  indexf2 = int( d3 / 5.0 + 0.5 ) + 1
			  rintens(indexf1,indexcf,indexf2)=
     1                         rintens(indexf1,indexcf,indexf2)+grwt(i)
		       endif

 880		    end do
 900		 end do
!       so we've gone thru all the components; now evaluate what to do
!       with this grain
!       
!       print*,
!       print*, 'For grain number ',i
		 comp_index(i)=0 !  make sure
		 discomps=997.
		 totwt=0.
		 do ijk=1,numcomps
!       write(*,"('comp no, min ang',i4,f8.3)") ijk,dismin(ijk)
		    if(dismin(ijk).lt.discomps) then
		       discomps=dismin(ijk)
		       if(dismin(ijk).lt.tolerns) comp_index(i)=ijk
		    endif
		    if(dismin(ijk).lt.tolerns) then
		       if(dismin(ijk).gt.1.e-3) then
			  wtcomp(ijk)=1./(dismin(ijk)**4)
		       else
			  wtcomp(ijk)=1.e+12
		       endif
		       totwt=totwt+wtcomp(ijk)
!       write(*,"('comp no, wt.',i4,f8.3)") ijk,wtcomp(ijk)
		    endif
		 enddo
		 if(comp_index(i).ne.0) then
!       print*, 'Component ',comp_index(i),
!       &	' identified for grain number ',i
		 endif
		 if(totwt.gt.0.) then
		    do ijk=1,numcomps
		       if(dismin(ijk).lt.tolerns) then
			  wtcomp(ijk)=wtcomp(ijk)/totwt !  normalize
!       print*, 'comp no , wt ',ijk,wtcomp(ijk)
			  comp_vol(ijk)=comp_vol(ijk)
     $                         + (grwt(i)*wtcomp(ijk))
!       print*, 'Component name = ',comp_name(comp_index(i))
		       endif
		    enddo
		 endif
!       done with this grain (at last!)
!                 print*,'gr, equivs TRI, ORTHO: ',i,equivtri,equivorth

 1000	      end do
      
	      print*, 'Time = ',time
	      print*, 'Report on volume fractions..... '
              randpart = 1.0
	      do 1005, i = 1 , numcomps
		 write(*,
     $                "('For component ',a
     $                ,'  vol. frac.= ',f8.3,' %')") 
     1	      comp_name(i),100.*comp_vol(i)/sumvol
                 randpart = randpart - ( comp_vol(i)/sumvol )
 1005	      end do
              print* , ' random fraction = ' , randpart
	      write ( 2 , '( 2x , i12, 20(1x,f7.3) )' ) time
     1	   , ( comp_vol(i)/sumvol , i = 1 , numcomps )
!       pause
      
	      call normpf(polearray,ipolearray,hklgen
     $             , iavgpf,numpolfigs)	
	      call normpf(invpolearray,iinvpolearray
     $             ,invpfgen,iavginvpf,3)	
!  normalize the pole and inverse pole figures
!	      print*, 'done with normpf'

	      type = 'ppf'
	      call wpf(index_sod,only_one,type,polearray,ipolearray
     &             ,hklgen,iavgpf,numpolfigs) 
!  write out the pole figures

!	      call wpf_GMT(index,only_one) 
!  write out GMT formatted PFs: longitude, latitude, intensity
!	      write(*,*) 'done with wpf for use with GMT plotting'
!  implement this later on!

	      type = 'ipf'
!              print*,'IPF output disabled for now'
	      call wpf(index_sod,only_one,type
     $             , invpolearray,iinvpolearray
     &             ,invpfgen,iavginvpf,3) 
!  write out the inverse pole figures
!	      print*, 'done with wpf, ipf'

!	      call wpf_GMT(index,only_one) 
!  write out GMT formatted PFs: longitude, latitude, intensity
!	      write(*,*) 'done with wpf for use with GMT plotting'
!  implement this later on!

!  triclinic sample symm
	      irange = 4
	      call normod(irange)	!normalizes
	      call wod(index_sod,only_one,irange,Kocks) ! writes SOD file
	      call wodcod(index_sod,only_one,irange,Kocks) ! writes COD file
       
!  orthorhombic sample symm
	      irange = 1
	      call normod(irange)	!normalizes
	      call wod(index_sod,only_one,irange,Kocks) ! writes SOD file
	      call wodcod(index_sod,only_one,irange,Kocks) ! writes COD file
       
 8990	   continue
 9000	continue

        do j = 1 , numpolfigs
           close( unit = j+10 )
        enddo                   !  num. PFs
        do j = 1 , 3
           open( unit = j+20 )
        enddo                   !  num. IPFs
        
        print*,'_________'
        print*,'3 degr smoothing recommended for pole figures:'
        print*,'/code/popLA/popLA_ADR/smooth '
     &       ,fname(1:(index_sod))//'.ppf 3. n'
        print*,' ... similar for .ipf'
        print*
        print*,'Use this command to convert PF files to GMT format:'
        print*,
     $       '/code/popLA/Batchfiles.SVEN/pf2GMT '
     &       , fname(1:(index_sod))//'.mpf'
        print*,' and then this (mpf example) to generate plots:'
        print*,
     1       '/code/popLA/Batchfiles.SVEN/DrawStereograms.sh 3 '
     &       ,fname(1:(index_sod))//'_mpf',' PF'

        print*
        print*,'_________'
        print*,'For the .COD or .SOD files, try smoothing with:'
        print*,'/code/paul.lee.codes/smoothsod'
        print*
        print*,'then, plot it with '
        print*,'[/code/paul.lee.codes/]sodcon '
     &       , fname(1:(index_sod))//'.cmh 1 1 1'
	call exit
	end

!  ______________________________
!*****NEWDATE  /  borrowed from psplot.txt
      subroutine newdate(dat,tim)
      integer date_time(8)
      character*10 dummyarg
      character*(*)dat,tim
      character*36 monnam
      data monnam/'JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC'/

      call date_and_time(dummyarg,dummyarg,dummyarg,date_time)

!  Convert new date/time to the old way

      write(dat(1:2),'(i2.2)')date_time(3)
      mind=(date_time(2)-1)*3+1
      dat(4:6)=monnam(mind:mind+2)
      iyronly=date_time(1)-(date_time(1)/100)*100
      write(dat(8:9),'(i2.2)')iyronly
      dat(3:3)='-'
      dat(7:7)='-'

      write(tim(1:2),'(i2.2)')date_time(5)
      write(tim(4:5),'(i2.2)')date_time(6)
      write(tim(7:8),'(i2.2)')date_time(7)
      tim(3:3)=':'
      tim(6:6)=':'
      return
      end
!       
!  ------------------
!
      subroutine testangs(phi1, capphi, phi2,result)
!  tests to see if the angles fall within the specified range
!  which for now is just 0<=angle<=90  
!  changed it to accept 0. exactly, vii 09

!  LOCAL VARIABLES
      integer i
      real phi1,capphi,phi2
      logical result

!  CODE::
      result=.true.
      if(phi1.lt.0.0) result=.false.
      if(phi1.gt.90.0) result=.false.
      if(capphi.lt.0.0) result=.false.
      if(capphi.gt.90.0) result=.false.
      if(phi2.lt.0.0) result=.false.
      if(phi2.gt.90.0) result=.false.
      return
      end
!       
!  ------------------
!
	subroutine testangs_tri(phi1, capphi, phi2,result)
!  tests to see if the angles fall within the specified range
!  appropriate for the specified xtal symmetry and no sample symmetry

!  LOCAL VARIABLES

	implicit none

	integer i
	real phi1,capphi,phi2
	logical result

	result=.true.
	if(phi1.le.0.0) result = .false.
	if(phi1.gt.360.0) result = .false.
	if(capphi.le.0.0) result = .false.
	if(capphi.gt.90.0) result = .false.
	if(phi2.le.0.0) result = .false.
	if(phi2.gt.90.0) result = .false.
	return
	end
!       
!  ------------------
!
	subroutine testangs_hex(phi1, capphi, phi2,result)

!  tests to see if the angles fall within the specified range
!  which for now is just 0<angle<=90
!  HCP:  phi2 <= 60

!  LOCAL VARIABLES
	integer i
	real phi1,capphi,phi2
	logical result

! CODE::
	result = .true.
	if(phi1.le.0.0) result = .false.
	if(phi1.gt.90.0) result = .false.

	if(capphi.le.0.0) result = .false.
	if(capphi.gt.90.0) result = .false.

	if(phi2.le.0.0) result = .false.
	if(phi2.gt.60.0) result = .false.  !  HCP

	return
	end
!       
!  ------------------
!
	subroutine testangs_tri_hex(phi1, capphi, phi2,result)
!  tests to see if the angles fall within the specified range
!  appropriate for the specified xtal symmetry and no sample symmetry
!  HCP:  phi2 <= 60

!  LOCAL VARIABLES

	implicit none

	integer i
	real phi1,capphi,phi2
	logical result

!  CODE::
	result = .true.
	if(phi1.le.0.0) result = .false.
	if(phi1.gt.360.0) result = .false.
	if(capphi.le.0.0) result = .false.
	if(capphi.gt.90.0) result = .false.
	if(phi2.le.0.0) result = .false.
	if(phi2.gt.60.0) result = .false.  !  HCP
	return
	end
!       
!  ------------------
!
	subroutine testangs_ort ( phi1, capphi, phi2 , result )

!  tests to see if the angles fall within the specified range
!  which for now is just 0<angle<=90

!  LOCAL VARIABLES
	integer i
	real phi1,capphi,phi2
	logical result

! CODE::
	result = .true.
	if(phi1.le.0.0) result = .false.
	if(phi1.gt.90.0) result = .false.

	if(capphi.le.0.0) result = .false.
	if(capphi.gt.90.0) result = .false.

	if(phi2.le.0.0) result = .false.
	if(phi2.gt.90.0) result = .false.

	return
	end
!       
!  ------------------
!
	subroutine testangs_tri_ort(phi1, capphi, phi2,result)
!  tests to see if the angles fall within the specified range
!  appropriate for the specified xtal symmetry and no sample symmetry

!  LOCAL VARIABLES

	implicit none

	integer i
	real phi1,capphi,phi2
	logical result

	result=.true.
	if(phi1.le.0.0) result = .false.
	if(phi1.gt.360.0) result = .false.
	if(capphi.le.0.0) result = .false.
	if(capphi.gt.90.0) result = .false.
	if(phi2.le.0.0) result = .false.
	if(phi2.gt.90.0) result = .false.
	return
	end
!
!  ------------------
!
	subroutine normod(irange)
!  normalizes an SOD array
!  cells centered on the value are assumed (jw=kw=1 in popLA)
!  which means that the cells at 0 and 90 have to be treated 
!  differently from the rest

	implicit none

!  LOCAL VARIABLES
	integer i,j,k
	integer ns,is,iph,iran,nuphi1,nuphi2
	real cenph,phi1,capphi,phi2,oldcapphi,newcapphi,darea(19)
	real sum,rinmax,scale,dvol(73,19,19),voltot
!  DVOL contains the volume of each cell
	real xintens(73,19,19)
	real degrad

!  GLOBAL VARIABLES
	integer irange
	integer intens(73,19,19),imax,iavg
	real rintens(73,19,19),projs(73,19),cprojs(73,19),delph
	real triclinic(73,19,19)
	common /t1/ rintens,triclinic,projs,cprojs,intens,delph,iavg

	data degrad / 57.29578 /
        real sumweights  !  sum over all the weights 

!  An OD in units of Multiples of a Random Density  (MRD)
!                 or Multiples of a Uniform Density (MUD)
!  means that the integral of the intensity, multiplied by
!  the volume element {DVOL}, over the range of the space is 
!  equal to the volume of that space {VOLTOT}.
!  Note the contrast with a probability density function
!  for which the integral of the density times the volume
!  element is equal to unity, i.e. area under curve = 1
!  Thus, in a discretized space, the intensity associated
!  with each cell, multiplied by the integral of the volume
!  element for that cell, when summed over all cells, is 
!  also equal to the volume.
!  The weight fraction in each cell (probability of finding
!  an orientation in that range of angles) is equal to the 
!  intensity, times the cell volume, divided by the total vol.
 
!  Volume of each cell = delta-cosine(Phi) * 5 * 5, in this case
!  with corrections at the edges and corners
!  Since the integral of SINE over the range 0 to 90 = 1.,
!  the total volume equals the product of the range of the
!  1st and 3rd angles, so either 90*90, or 360*90 here.

!  To go from volume fractions to intensities, we must
!  reverse the process.
!  The intensity in each cell is equal to the weight, divided
!  by the total weight, divided by the cell volume, and
!  multiplied by the total volume (of the space).

!  To test all this, generate a list of orientations on a
!  regular lattice of points in Euler angle space, but
!  be careful to assign a weight to each point (line) that
!  correponds to the volume of the corresponding cell!

!  notes added vii 09, ADR

!  CODE::
	if(irange.ne.1.and.irange.ne.4) stop 'wrong IRANGE'

        if(irange.eq.1) then
           voltot = 90.*90.
        elseif(irange.eq.4) then
           voltot = 360. * 90.
        endif

        sum = 0.
	oldcapphi = 0.
	do 100, i = 1,19
           capphi = float(i-1)*5.0
           newcapphi = capphi+2.5
           newcapphi = amin1(newcapphi,90.0)
           darea(i) = cos(oldcapphi/degrad)-cos(newcapphi/degrad)
           oldcapphi = newcapphi
           sum = sum + darea(i)
100	end do
!$$$	print*, 'completed 100 loop; DAREA(): ',darea
!$$$        print*, ' sum = ', sum, ', = 1.0 ??'

        sumweights = 0.
	nuphi2 = 19
	if(irange.eq.1) then
	   nuphi1 = 19
	   do i = 1,nuphi1
	      do j = 1,19
		 do k = 1,nuphi2
		    sumweights = sumweights + rintens(i,j,k)
		 enddo
	      enddo
	   enddo
	elseif(irange.eq.2) then
	   nuphi1 = 37
	   do i = 1,nuphi1
	      do j = 1,19
		 do k = 1,nuphi2
		    sumweights = sumweights + triclinic(i,j,k)
		 enddo
	      enddo
	   enddo
	elseif(irange.eq.4) then
	   nuphi1 = 73
	   do i = 1,nuphi1
	      do j = 1,19
		 do k = 1,nuphi2
		    sumweights = sumweights + triclinic(i,j,k)
		 enddo
	      enddo
	   enddo
	endif
!        print*,'NORMOD: range, phi1-max =  ',irange,nuphi1
!        print*,'NORMOD: total weight = ', sumweights

	sum = 0.0
	do 200, i = 1,nuphi1
	   do 190, j = 1,19
	      do 180, k = 1,nuphi2
!		 dvol(i,j,k) = darea(j)*5.*5./voltot
		 dvol(i,j,k) = darea(j) * 5. * 5.
		 if(i.eq.1.or.i.eq.nuphi1) dvol(i,j,k)=dvol(i,j,k)*0.5
		 if(k.eq.1.or.k.eq.nuphi2) dvol(i,j,k)=dvol(i,j,k)*0.5
		 sum = sum + dvol(i,j,k)
!     deal with the edge and corner cells
 180	      end do
 190	   end do
200	end do
!	print*
!     $       ,'NORMOD: 200 loop; VOL=voltot?  vol. sum=',sum,voltot

	sum = 0.0
	do 1000, i = 1,nuphi1
	   do j = 1,19
	      do k = 1,nuphi2
                 if(irange.eq.1) then
                    xintens(i,j,k) = voltot * rintens(i,j,k)
     &                   / dvol(i,j,k) / sumweights
                 elseif( irange.eq.2 .or. irange.eq.4 ) then
!  required to transfer binned intensities to XINTENS - 6 Sep 09
                    xintens(i,j,k) = voltot * triclinic(i,j,k)
     &                   / dvol(i,j,k) / sumweights
                 end if
!		 sum = sum + xintens(i,j,k)
		 sum = sum + xintens(i,j,k) * dvol(i,j,k)
!  should always use the volume increment to compute the sum
              end do
           end do
 1000	end do
!	print*, 'NORMOD: 1000 loop; SUM,voltot= ',sum,voltot

	if(sum.eq.0.0) stop 'failed - no intensity?!'
!	scale = 1.0 / sum
	scale = voltot / sum
        print*,'NORMOD: scale [for intensities] = ' , scale
     $       , ' ... should be = 1.'
	rinmax = 0.0
	do 2000, i = 1,nuphi1
	   do j = 1,19
	      do k = 1,nuphi2
!		 xintens(i,j,k) = scale*xintens(i,j,k)/dvol(i,j,k)
		 xintens(i,j,k) = scale * xintens(i,j,k)
		 if(xintens(i,j,k).gt.rinmax) rinmax = xintens(i,j,k)
              end do
           end do
 2000	end do
!$$$	print*, 'NORMOD: 2000 loop; RINMAX= ',rinmax

	scale = 1.0
	iavg = 100
	if(rinmax.gt.99.98) then
	   scale = scale * 99.98 / rinmax
	   iavg = max0( 1 , int(9998./rinmax) )
	endif
!     this brings everything back down into i4 format!
!$$$	print*,'NORMOD: scale factor [1.0], IAVG = ',scale,iavg
	rinmax = 0.0
	do 3000, i = 1,nuphi1
	   do j = 1,19
	      do k = 1,19
		 xintens(i,j,k) = scale*xintens(i,j,k)
		 intens(i,j,k) = int(100.0*xintens(i,j,k))
		 if(xintens(i,j,k).gt.rinmax) rinmax = xintens(i,j,k)
              enddo
           enddo
 3000	end do
	if(rinmax.gt.9999.) stop 'error in scaling!'
!$$$	pause 'hit return to continue'
!$$$c  DEBUG LINES ->
!$$$	do 4000, i = 1 , nuphi1
!$$$           print*, 'NORMOD: Block for phi1 = ',(i-1)*5.
!$$$           do 3900, j=1,19
!$$$              write(*,3800) (intens(i,j,k),k=1,19)
!$$$ 3800         format(1x,19i5)
!$$$ 3900      end do
!$$$ 4000   end do
!	pause

!	print*, 'Real Numbers:'
!	do 5000, i=1,19
!		print*, 'Block for phi1 = ',(i-1)*5.
!		do 4900, j=1,19
!			write(*,4800) (xintens(i,j,k),k=1,19)
!4800		format(1x,19f6.1)
!4900		continue
!5000	continue
!	pause

!  now to deal with the projections

!  first the SOP
	do  j=1,19
	   do  k=1,19
	      projs(j,k)=0.0
	      do  is = 1,nuphi1
		 projs(j,k)=projs(j,k)+xintens(is,j,k)
	      enddo
	      projs(j,k)=projs(j,k)/float(nuphi1)
	   enddo
	enddo
!  and now the COP
	do  j=1,nuphi1
	   do  k=1,19
	      cprojs(j,k)=0.0
	      do is = 1,19
		 cprojs(j,k)=cprojs(j,k)+xintens(j,k,is)
	      enddo
	      cprojs(j,k)=cprojs(j,k)/19.
	   enddo
	enddo

!  DEBUG LINES ->
!	print*, 'Check SOP'
!	do 2000, i=1,19
!		write(*,1800) (projs(i,j),j=1,19)
!1800		format(1x,19f4.1)
!2000	continue

!	print*, 'Check COP'
!	do 3000, i=1,19
!		write(*,1800) (cprojs(i,j),j=1,19)
!3000	continue

	return
	end

!  ------------------

	subroutine normpf(polearray,ipolearray,hklgen,iavgpf,numpolfigs)
!  normalizes an Pole Figure array
!  cells centered on the value are assumed (jw=kw=1 in popLA)
!  which means that the cells at 0 and 90 have to be treated 
!  differently from the rest

!  LOCAL VARIABLES
	integer ns,is,iph,iran,nuphi1,nuphi2
	real cenph,phi1,capphi,phi2,oldcapphi,newcapphi,darea(19)
	real sum,pfmax(10),scale,dvol(73,19,19),voltot
	real sumpf(10)
!  DVOL contains the volume of each cell
!
!  GLOBAL VARIABLES
	integer intens(73,19,19),imax,iavg
	real rintens(73,19,19),projs(73,19),cprojs(73,19),delph
	real hklgen(3,10),check(10)  !  allow for 10 pole figures
	integer numpolfigs
	real polearray(73,19,10)  !  accumulator for pole fig
	integer ipolearray(73,19,10),iavgpf(10)  ! integer form of pole fig
	real triclinic(73,19,19)

	common /t1/ rintens,triclinic,projs,cprojs,intens,delph,iavg
!	common /t1/ rintens,projs,cprojs,intens,delph,iavg
!	common /poles/ polearray,ipolearray,hklgen,iavgpf,numpolfigs
!  changed from common to arguments so as to re-use for IVPs

!  CODE::
	degrad=57.29578

	oldcapphi=0.
	do i=1,19
	   capphi = float(i-1)*5.0
	   newcapphi = capphi+2.5
	   newcapphi = amin1(newcapphi,90.0)
	   darea(i) = cos(oldcapphi/degrad)-cos(newcapphi/degrad)
	   oldcapphi=newcapphi
100	enddo
!	print*, 'completed 100 loop; DAREA()= ',darea

	sum = 0.0
	nuphi1 = 73		!changed for full pole figures
	nuphi2 = 19
	voltot = 360.
	do 200, i = 1,nuphi1
	   do 190, j = 1,nuphi2
	      k=1
	      dvol(i,j,k) = darea(j)*5./voltot
	      if(i.eq.1.or.i.eq.nuphi1) dvol(i,j,k)=dvol(i,j,k)*0.5
!	      if(j.eq.1.or.j.eq.nuphi2) dvol(i,j,k)=dvol(i,j,k)*0.5
!  this was dealt with above 
	      sum=sum+dvol(i,j,k)
!  deal with the edge and corner cells
 190	   enddo
 200	enddo
!$$$	print*, 'completed 200 loop; check VOL=1?  vol.sum=',sum

!	print*, 'no. of pole figures =',numpolfigs
	do i=1,numpolfigs	!make this 72 for no sample symm
	   sumpf(i)=0.0
	   do j=1,72
	      do k=1,19
		 sumpf(i) = sumpf(i)+polearray(j,k,i)
 	      enddo
 	   enddo
 	enddo
!$$$	print*, 'completed 1000 loop; SUM= ',(sumpf(i),i=1,numpolfigs)

	do i=1,numpolfigs		!make this 72 for no sample symm
	   if(sumpf(i).eq.0.0) stop 'failed - no intensity?!'
	   scale = 1.0/sumpf(i)
	   pfmax(i)=0.0
	   do  j=1,72
	      do  k=1,19
		 polearray(j,k,i)=scale*polearray(j,k,i)/dvol(j,k,1)
		 if(polearray(j,k,i).gt.pfmax(i)) then
                    pfmax(i)=polearray(j,k,i)
                 endif
 	      enddo
 	   enddo
!$$$           print*, 'For PF #',i,' MAX value = ',pfmax(i)
 2000	enddo

        do i = 1,numpolfigs
           check(i) = 0.
           do j = 1,72
              do k = 1,19
                 check(i) = check(i)+polearray(j,k,i)*dvol(j,k,1)
              enddo
           enddo
!$$$           print*,'check sum= for PF #',i,' = ',check(i),' =1?'
        enddo

	do i=1,numpolfigs		
	   scale=1.0
	   iavgpf(i)=100
	   if(pfmax(i).gt.99.98) then
	      scale=scale*99.98/pfmax(i)
	      iavgpf(i)=max0(1,int(9998./pfmax(i)))
	   endif
!  this brings everything back down into i4 format!
	   pfmax(i)=0.0
	   do j=1,72
	      do k=1,19
		 polearray(j,k,i)=scale*polearray(j,k,i)
		 ipolearray(j,k,i)=int(100.0*polearray(j,k,i))
		 if(polearray(j,k,i).gt.pfmax(i)) then
                    pfmax(i)=polearray(j,k,i)
                 endif
!	print*, 'j,k,polearray,ipolearray '
!     &   ,j,k,polearray(j,k,i),ipolearray(j,k,i)
 2800	      enddo
 2900	   enddo
3000	enddo
	do i=1,numpolfigs
	   if(pfmax(i).gt.9999.) stop 'error in scaling!'
	enddo

!	pause 'hit return to continue'
!  DEBUG LINES ->
!	do 4000, i=1,19
!		print*, 'Block for phi1 = ',(i-1)*5.
!		do 3900, j=1,19
!			write(*,3800) (intens(i,j,k),k=1,19)
!3800		format(1x,19i4)
!3900		continue
!4000	continue
!	pause 'hit return to continue'
!
!	print*, 'Real Numbers:'
!	do 5000, i=1,19
!		print*, 'Block for phi1 = ',(i-1)*5.
!		do 4900, j=1,19
!			write(*,4800) (rintens(i,j,k),k=1,19)
!4800		format(1x,19f4.1)
!4900		continue
!5000	continue

	return
	end

!
!  ------------------
!
	subroutine sop_calc
!  calculates the SOP projection
!  from RINTENS into PROJS
!
!  LOCAL VARIABLES
	integer ns,is,iph,iran
	real cenph
	integer iper(3)
!
!  GLOBAL VARIABLES
	character hkl*3,seclab*5,title*8,idate*8
	integer intens(73,19,19),imax,iavg
	character prosa*80,prosa1*80,hh*1,kk*1,ll*1,lper,nomon
	character eulan*25,nomen		! IPER is different in WOD
	character*5 label,secid
	character fname*50
	real rintens(73,19,19),projs(73,19),cprojs(73,19),delph
	character*80 textitl
	integer numsymm,numcsl,nsig,ngrain,numsamp
	real triclinic(73,19,19)

	common/a1/ quatsymm(4,48),numsymm,quatsamp(4,4),numsamp
	common/a2/quatcsl(4,50),numcsl,nsig(50)
	common/a3/aquat(4,2000000),ngrain,phi1(2000000)
     &	,capphi(2000000),phi2(2000000)
	common/a4/ grwt(2000000),eulan

        logical add_90
	common/strings/ fname,hkl,seclab,title,idate,textitl,add_90
	common /t1/ rintens,triclinic,projs,cprojs,intens,delph,iavg
!	common /t1/ rintens,projs,cprojs,intens,delph,iavg
!
1	format(2a)
    2 format(i1)
    8 format(a)
!    8 format(a\)
  111 format(a1,73x,3i2)
!
	degrad=57.29578
	iper(1)=1
	iper(2)=2
	iper(3)=3
	delph=5.0
	secid=' '//'SOP'//'B'
	seclab='PROJ='

	ns=19
!  assume 19 sections plus a projection for now
!  so, this version no longer usable for general sample symmetry 
	do 1312 j=1,19
	   do 1200, k=1,19
	      projs(j,k)=0.0
	      cprojs(j,k)=0.0
	      do 1000 is=1,ns
		 projs(j,k)=projs(j,k)+rintens(is,j,k)
		 cprojs(j,k)=cprojs(j,k)+rintens(k,j,is)
 1000	      continue
	      projs(j,k)=projs(j,k)/float(ns)
	      cprojs(j,k)=cprojs(j,k)/float(ns)
 1200	   continue
 1312	continue
!
!  DEBUG LINES ->
!	print*, 'Check SOP'
!	do 2000, i=1,19
!		write(*,1800) (projs(i,j),j=1,19)
1800		format(1x,19f4.1)
!2000	continue
!
!	print*, 'Check COP'
!	do 3000, i=1,19
!		write(*,1800) (cprojs(i,j),j=1,19)
!3000	continue
!
	return
	end

!
!  ------------------
!
	subroutine wpf(pointx,only_one,type,polearray
     $     ,ipolearray,hklgen,iavgpf,numpolfigs)
!  writes out an pole figure file, or inverse pole figure file

!  LOCAL VARIABLES
	integer n,ns,is,iph,iran,pointx
	real cenph,degrad
	integer iper(3)
	logical only_one

!  GLOBAL VARIABLES
	character hkl*3,seclab*5,title*8,idate*8
	integer intens(73,19,19),imax,iavg
	character prosa*80,prosa1*80,hh*1,kk*1,ll*1,lper,nomon
	character eulan*25,nomen		! IPER is different in WOD
	character*5 label,secid
	character fname*50
	real rintens(73,19,19),projs(73,19),cprojs(73,19),delph
	character*80 textitl
	integer numsymm,numcsl,nsig,ngrain,numsamp
	character type*3

	real hklgen(3,10)  !  allow for 10 pole figures
	real polearray(73,19,10)  !  accumulator for pole fig
	integer ipolearray(73,19,10),iavgpf(10)  ! integer form of pole fig
	integer numpolfigs
	real triclinic(73,19,19)

	common/a1/ quatsymm(4,48),numsymm,quatsamp(4,4),numsamp
	common/a2/quatcsl(4,50),numcsl,nsig(50)
	common/a3/aquat(4,2000000),ngrain,phi1(2000000)
     &	,capphi(2000000),phi2(2000000)
	common/a4/ grwt(2000000),eulan

        logical add_90
	common/strings/ fname,hkl,seclab,title,idate,textitl,add_90
	common /t1/ rintens,triclinic,projs,cprojs,intens,delph,iavg
!	common /t1/ rintens,projs,cprojs,intens,delph,iavg
!	common /poles/ polearray,ipolearray,hklgen,iavgpf,numpolfigs

!  CODE::
1	format(2a)
2	format(i1)
8	format(a)
!    8 format(a\)
111	format(a1,73x,3i2)

!	print*,'Entering WPF with type = ',type

	degrad=57.29578
	iper(1)=1
	iper(2)=2
	iper(3)=3
	delph=5.0
	seclab='PROJ '
	label='     '

!	print*, 'Check IAVG: ',iavg

	if(only_one) then
	   if(type.eq.'ppf') then
	      open(7,file=fname(1:(pointx))//'.ppf',status='unknown')
	   elseif(type.eq.'ipf') then
	      open(7,file=fname(1:(pointx))//'.ipf',status='unknown')
	   endif
	else
	   if(type.eq.'ppf') then
	      open(7,file=fname(1:(pointx+3))//'.ppf',status='unknown')
	   elseif(type.eq.'ipf') then
	      open(7,file=fname(1:(pointx+3))//'.ipf',status='unknown')
	   endif
	endif

	nseg=72  ! 4 rows, 18 in each row
	do n=1,numpolfigs  !  1000
!	   print*, 'writing pole figure no. ',n
!  sort out the labels
	   ns=hklgen(1,n)
	   hh=char(48+ns)
	   ns=hklgen(2,n)
	   kk=char(48+ns)
	   ns=hklgen(3,n)
	   ll=char(48+ns)
	   secid='('//hh//kk//ll//')'
	   write(7,8) textitl
	   if(ngrain.lt.99999) then
	      write(7,1332) secid,5.0,90.0,5.0,360.0,1,1
!       write(7,1332) secid,delth,rm,delom,pm,iw,jw
     &	,(iper(i),i=1,3),iavgpf(n),ngrain,seclab,label
	   else
	      write(7,1332) secid,5.0,90.0,5.0,360.0,1,1
     &	,(iper(i),i=1,3),iavgpf(n),1,seclab,label
!  have to fudge if dealing with a very large set of grains, e.g. from OIM
1332	format(a5,4f5.1,5i2,2i5,2a5)
	endif

	mseg=19
	do m=1,mseg
	   write(7,1318) (ipolearray(is,m,n),is=1,nseg)
	enddo
 1318 format(1x,18i4)
 1319 format(1x,19i4)
      write(7,*)

!*         ********** (commented out:) ring intensities:
!*      if(is.lt.ns) goto 1350
!*      write(7,1) ' '
!*      write(7,1332) secid,delth,rm,delom,pm,iw,jw
!*     &,(iper(i),i=1,3),iavg,ngr,seclab,label
!*      write(7,1335)
!* 1335 format('ring centered at; avg.grain fraction;  std.dev.:')
!*      do 1359 n=1,nrad
!*      centh=delth*(n-.5-iw*.5)
!* 1359 write(7,1333) centh,frad(n),fdev(n)
!*      write(7,*)
!* 1350 continue
!* 1333 format(3f8.5)
!****
1000	enddo

	close(7)
	return
	end
!
!  ------------------
!
	subroutine wpf_GMT(pointx,only_one,type,polearray
     $     ,ipolearray,hklgen,iavgpf,numpolfigs)
!  writes out an pole figure file
!  in GMT format, longitude, latitude, intensity
!  file extension = GPF
!
!  LOCAL VARIABLES
	integer n , ns , is , iph , iran , pointx
	real cenph,degrad
	integer iper(3)
	logical only_one
!
!  GLOBAL VARIABLES
	character hkl*3,seclab*5,title*8,idate*8
	integer intens(73,19,19),imax,iavg
	character prosa*80,prosa1*80,hh*1,kk*1,ll*1,lper,nomon
	character eulan*25,nomen		! IPER is different in WOD
	character*5 label,secid
	character fname*50
	real rintens(73,19,19),projs(73,19),cprojs(73,19),delph
	character*80 textitl
	integer numsymm,numcsl,nsig,ngrain,numsamp
!
	real hklgen(3,10)  !  allow for 10 pole figures
	real polearray(73,19,10)  !  accumulator for pole fig
	integer ipolearray(73,19,10),iavgpf(10)  ! integer form of pole fig
	integer numpolfigs
	real triclinic(73,19,19)
        character one*1   !  for file name index

	common/a1/ quatsymm(4,48),numsymm,quatsamp(4,4),numsamp
	common/a2/quatcsl(4,50),numcsl,nsig(50)
	common/a3/aquat(4,2000000),ngrain,phi1(2000000)
     &	,capphi(2000000),phi2(2000000)
	common/a4/ grwt(2000000),eulan

        logical add_90
	common/strings/ fname,hkl,seclab,title,idate,textitl,add_90
!	common /t1/ rintens,triclinic,projs,cprojs,intens,delph,iavg
!	common /t1/ rintens,projs,cprojs,intens,delph,iavg
!	common /poles/ polearray,ipolearray,hklgen,iavgpf,numpolfigs

!  CODE::
1	format(2a)
2	format(i1)
8	format(a)
!    8 format(a\)
111	format(a1,73x,3i2)

	degrad=57.29578
	iper(1)=1
	iper(2)=2
	iper(3)=3
	delph=5.0
	seclab='PROJ '
	label='     '

!	write(*,*) 'Check IAVG: ',iavg

!	write(*,*) ' Enter the name for the output file (xxx.sod): '
!        read(*,*) fname
	nseg=72  ! 4 rows, 18 in each row
	do n=1,numpolfigs  !  1000
           one=char(48+n)
           write(*,*) 'writing pole figure no. ',n
           if(only_one) then
              open(7,file=fname(1:(pointx))//'_'//one//'.gpf'
     &             ,status='unknown')
           else
              open(7,file=fname(1:(pointx+3))//'_'//one//'.gpf'
     &             ,status='unknown')
           endif

!  sort out the labels
           ns=hklgen(1,n)
           hh=char(48+ns)
           ns=hklgen(2,n)
           kk=char(48+ns)
           ns=hklgen(3,n)
           ll=char(48+ns)
           secid='('//hh//kk//ll//')'
!           write(7,8) textitl
           if(ngrain.lt.99999) then
              write(7,1332) secid,5.0,90.0,5.0,360.0,1,1
!		write(7,1332) secid,delth,rm,delom,pm,iw,jw
     &             ,(iper(i),i=1,3),iavgpf(n),ngrain,seclab,label
           else
              write(7,1332) secid,5.0,90.0,5.0,360.0,1,1
     &             ,(iper(i),i=1,3),iavgpf(n),1,seclab,label
!  have to fudge if dealing with a very large set of grains, e.g. from OIM
 1332         format(a5,4f5.1,5i2,2i5,2a5)
           endif
!
           mseg = 19
           do m = 1,mseg
              do is = 1,nseg
                 write(7,*) 5.*float(is-1),'  '
     $                ,5.*float(mseg-m),' '  !  19-m gives geog. latitude
     $                ,float(ipolearray(is,m,n))/100.
              enddo
           enddo
 1318      format(1x,18i4)
 1319      format(1x,19i4)
           close(7)
 1000   enddo
        return
        end
!
!  ------------------
!
	subroutine wod(pointx,only_one,irange,Kocks)
!  writes out an SOD file

!  LOCAL VARIABLES
	integer ns,is,iph,iran , pointx
	real cenph
	integer iper(3)
	logical only_one
	integer irange   !  signals range of phi1

!  GLOBAL VARIABLES
	character hkl*3,seclab*5,title*8,idate*8
	integer intens(73,19,19),imax,iavg
	character prosa*80,prosa1*80,hh,kk,ll,lper,nomon
	character eulan*25,nomen		! IPER is different in WOD
	character*5 label,secid
	character fname*50
	real rintens(73,19,19),projs(73,19),cprojs(73,19),delph
	character*80 textitl
	integer numsymm,numcsl,nsig,ngrain,numsamp
	real triclinic(73,19,19)
	logical Kocks
!
	common/a1/ quatsymm(4,48),numsymm,quatsamp(4,4),numsamp
	common/a2/quatcsl(4,50),numcsl,nsig(50)
	common/a3/aquat(4,2000000),ngrain,phi1(2000000)
     &	,capphi(2000000),phi2(2000000)
	common/a4/ grwt(2000000),eulan

        logical add_90
	common/strings/ fname,hkl,seclab,title,idate,textitl,add_90
	common /t1/ rintens,triclinic,projs,cprojs,intens,delph,iavg
!	common /t1/ rintens,projs,cprojs,intens,delph,iavg
!
1	format(2a)
    2 format(i1)
    8 format(a)
!    8 format(a\)
  111 format(a1,73x,3i2)

      print*,'Entering WOD to print out SOD'

      degrad=57.29578
      iper(1)=1
      iper(2)=2
      iper(3)=3
      delph=5.0
      secid=' '//'SOD'//'B'
      seclab='Phi1='

	if(Kocks) then
	   secid=' '//'SOD'//'K'
	   seclab='PSI ='
	endif
!	print*, 'Check IAVG: ',iavg

!	print*, ' Enter the name for the output file (xxx.sod): '
!        read(*,*) fname

!  DEBUG LINES ->
!	do i=1,19
!	do i=1,1
!	   print*, 'Block for phi1 = ',(i-1)*5.
!	   do  j=1,19
!	      write(*,"(1x,19i5)") (intens(i,j,k),k=1,19)
!	   enddo
!	enddo
!	pause 'debug print out of INTENS'

	if(irange.eq.1) then
	   if(only_one) then
	      open(7,file=fname(1:(pointx))//'.sod',status='unknown')
	   else
	      open(7,file=fname(1:(pointx+3))//'.sod',status='unknown')
	   endif
!  next, shear textures
	elseif(irange.eq.2) then
	   if(only_one) then
	      open(7,file=fname(1:(pointx))//'_180.sod'
     $             ,status='unknown')
	   else
	    open(7,file=fname(1:(pointx+3))//'_180.sod'
     $             ,status='unknown')
	   endif
!  next triclinic sample symmetry
	elseif(irange.eq.4) then
	   if(only_one) then
	      open(7,file=fname(1:(pointx))//'_360.sod'
     $             ,status='unknown')
	   else
	    open(7,file=fname(1:(pointx+3))//'_360.sod'
     $             ,status='unknown')
	   endif
	else
	   stop 'invalid value of irange'
	endif

!	ns=20
	ns = 18*irange + 2
!  number of sections determined by range of phi1 
!  plus a projection
	do 1000 is=1,ns
      if(is.eq.ns) then
        iph=1
        label='PROJ '
        iran=100
      else
        iph=is
        cenph=float(iph-1)*delph
        write(label,'(f5.1)') cenph
        iran=100*nph
      endif
!
	write(7,8) textitl
	if(ngrain.le.99999) then
		write(7,1332) secid,5.0,90.0,5.0,90.0,1,1
!      write(7,1332) secid,delth,rm,delom,pm,iw,jw
     &,(iper(i),i=1,3),iavg,ngrain,seclab,label
	else
		write(7,1332) secid,5.0,90.0,5.0,90.0,1,1
     &,(iper(i),i=1,3),iavg,1,seclab,label
 1332 format(a5,4f5.1,5i2,2i5,2a5)
!  have to fudge for v large data sets, e.g. OIM
	endif

	do 1312 n=1,19
	   mseg=19		! change this for lower sample symmetry
!      do 1312 n=1,nrad
!      do 1313 m=1,mseg
! 1313 if(intens(is,m,n).lt.1) intens(is,m,n)=1
!cccc                     to avoid 0 (taking logs in plotting routine)
	   nseg=(mseg/18-1)*18
	   
	   if(is.eq.ns) then	! test for PROJ
	      if(nseg.eq.0) goto 1420
	      write(7,1318) (int(100.*projs(n,m)),m=1,nseg)
 1420	      write(7,1319) (int(100.*projs(n,m)),m=nseg+1,mseg)

	   else
	      if(nseg.eq.0) goto 1320
	      write(7,1318) (intens(is,n,m),m=1,nseg)
 1320	      write(7,1319) (intens(is,n,m),m=nseg+1,mseg)
	   endif
 1312	continue
 1318 format(1x,18i4)
 1319 format(1x,19i4)
      write(7,*)
!*         ********** (commented out:) ring intensities:
!*      if(is.lt.ns) goto 1350
!*      write(7,1) ' '
!*      write(7,1332) secid,delth,rm,delom,pm,iw,jw
!*     &,(iper(i),i=1,3),iavg,ngr,seclab,label
!*      write(7,1335)
!* 1335 format('ring centered at; avg.grain fraction;  std.dev.:')
!*      do 1359 n=1,nrad
!*      centh=delth*(n-.5-iw*.5)
!* 1359 write(7,1333) centh,frad(n),fdev(n)
!*      write(7,*)
!* 1350 continue
!* 1333 format(3f8.5)
!****
 1000 continue
      close(7)
      return
      end
!
!  ------------------
!     
      subroutine wodcod(pointx,only_one,irange,Kocks)
!  writes out an COD file

      implicit none

!  LOCAL VARIABLES
      integer i,j,k,pointx
      integer ns,is,iph,iran
      real cenph
      integer iper(3)
      logical only_one
      integer irange
      real degrad
      integer nph,mseg,nseg,n,m
      real rrange

!  GLOBAL VARIABLES
      character hkl*3,seclab*5,title*8,idate*8
      integer intens(73,19,19),imax,iavg
      character prosa*80,prosa1*80,hh,kk,ll,lper,nomon
      character eulan*25,nomen  ! IPER is different in WOD
      character*5 label,secid
      character fname*50
      real rintens(73,19,19),projs(73,19),cprojs(73,19),delph
      character*80 textitl
      integer numsymm,ngrain,numsamp
      real triclinic(73,19,19)
      real quatsymm(4,48),quatsamp(4,4),quatcsl(4,50)
      integer numcsl,nsig(50)
      real aquat(4,2000000),phi1(2000000)
     &     ,capphi(2000000),phi2(2000000)
      real grwt(2000000)
      logical Kocks

      common/a1/ quatsymm,numsymm,quatsamp,numsamp
      common/a2/quatcsl,numcsl,nsig
      common/a3/aquat,ngrain,phi1,capphi,phi2
      common/a4/ grwt,eulan

      logical add_90
      common/strings/ fname,hkl,seclab,title,idate,textitl,add_90
      common /t1/ rintens,triclinic,projs,cprojs,intens,delph,iavg
!	common /t1/ rintens,projs,cprojs,intens,delph,iavg
!
 1    format(2a)
    2 format(i1)
    8 format(a)
!    8 format(a\)
  111 format(a1,73x,3i2)

      print*,'Entering WODCOD to print out COD'

      degrad=57.29578
      iper(1)=1
      iper(2)=2
      iper(3)=3
      delph=5.0
      secid=' '//'COD'//'B'
      seclab='Phi2='

	if(Kocks) then
	   secid=' '//'COD'//'K'
	   seclab='PHI ='
	endif
!  DEBUG LINES ->
      goto 3172
!     do i=1,19
      do i=1,1
	 print*, 'Block for phi1 = ',(i-1)*5.
	 do  j=1,19
	    write(*,"(1x,19i5)") (intens(i,j,k),k=1,19)
	 enddo
      enddo
 3172 continue
!     pause 'debug print out of INTENS'

!     print*, 'Check IAVG: ',iavg

!     print*, ' Enter the name for the output file (xxx.cod): '
!     read(*,*) fname

      if(irange.eq.1) then
	 if(only_one) then
	    open(7,file=fname(1:(pointx))//'.cod',status='unknown')
	 else
	    open(7,file=fname(1:(pointx+3))//'.cod',status='unknown')
	 endif
!  next, shear textures
      elseif(irange.eq.2) then
	 if(only_one) then
	    open(7,file=fname(1:(pointx))//'_180.cod',status='unknown')
	 else
	    open(7,file=fname(1:(pointx+3))//'_180.cod'
     $           ,status='unknown')
	 endif
!  next triclinic sample symmetry
      elseif(irange.eq.4) then
	 if(only_one) then
	    open(7,file=fname(1:(pointx))//'_360.cod',status='unknown')
	 else
	    open(7,file=fname(1:(pointx+3))//'_360.cod'
     $           ,status='unknown')
	 endif
      else
	 stop 'invalid value of irange'
      endif
      ns=20
!  assume 19 sections plus a projection for now
      do 1000 is=1,ns
	 if(is.eq.ns) then
	    iph=1
	    label='PROJ '
	    iran=100
	 else
	    iph=is
	    cenph=float(iph-1)*delph
	    write(label,'(f5.1)') cenph
	    iran=100*nph
	 endif
	 
	 rrange = float(irange)*90.
	 write(7,8) textitl
	 if(ngrain.le.99999) then
	    write(7,1332) secid,5.0,90.0,5.0,rrange,1,1
!     write(7,1332) secid,delth,rm,delom,pm,iw,jw
     &           ,(iper(i),i=1,3),iavg,ngrain,seclab,label
	 else
	    write(7,1332) secid,5.0,90.0,5.0,rrange,1,1
     &           ,(iper(i),i=1,3),iavg,1,seclab,label
 1332	    format(a5,4f5.1,5i2,2i5,2a5)
!     have to fudge for v large data sets, e.g. OIM
	 endif

!	   mseg=19		! change this for lower sample symmetry
	 mseg = 18*irange + 1
	 nseg = (mseg/18-1)*18
	   
	 do 1312 n=1,19
!     do 1312 n=1,nrad
!     do 1313 m=1,mseg
! 1313 if(intens(is,m,n).lt.1) intens(is,m,n)=1
!cccc                     to avoid 0 (taking logs in plotting routine)
	    if(is.eq.ns) then	! test for PROJ
	       if(nseg.eq.0) goto 1420
!$$$	       write(7,1318) (int(100.*cprojs(n,m)),m=1,nseg)
!$$$ 1420	       write(7,1319) (int(100.*cprojs(n,m)),m=nseg+1,mseg)
!  jul 08 - discovered that the order (n,m) should be (m,n)
	       write(7,1318) (int(100.*cprojs(m,n)),m=1,nseg)
 1420	       write(7,1319) (int(100.*cprojs(m,n)),m=nseg+1,mseg)
	       
	    else
	       if(nseg.eq.0) goto 1320
	       write(7,1318) (intens(m,n,is),m=1,nseg)
 1320	       write(7,1319) (intens(m,n,is),m=nseg+1,mseg)
	    endif
 1312	 continue
 1318	 format(1x,18i4)
 1319	 format(1x,19i4)
	 write(7,*)
!*         ********** (commented out:) ring intensities:
!*      if(is.lt.ns) goto 1350
!*      write(7,1) ' '
!*      write(7,1332) secid,delth,rm,delom,pm,iw,jw
!*     &,(iper(i),i=1,3),iavg,ngr,seclab,label
!*      write(7,1335)
!* 1335 format('ring centered at; avg.grain fraction;  std.dev.:')
!*      do 1359 n=1,nrad
!*      centh=delth*(n-.5-iw*.5)
!* 1359 write(7,1333) centh,frad(n),fdev(n)
!*      write(7,*)
!* 1350 continue
!* 1333 format(3f8.5)
!****
 1000 continue
      close(7)
      return
      end
!
! ________________________________________
!
!
      subroutine textur4(iq_symm)

!  second version, for use in REXGBS, WTS2POP
!  not for use in REX2D

      integer numsymm,numsamp
      integer iq_symm,n_max
      logical found

      common/a1/ quatsymm(4,48),numsymm,quatsamp(4,4),numsamp

! CODE::
      rad = 57.29578
      
      if(iq_symm.lt.1 .or. iq_symm.gt.3) stop 'bad symmetry class?'

      if(iq_symm.eq.1) then
         n_max = 24
         inquire ( file = 'quat.symm.cubic' , exist=found )
         if ( .not.found ) then
            print * , 'Did not find the symmetry file, quat.symm.cubic'
            call exit ( 1 )
         end if
      elseif(iq_symm.eq.2) then
         n_max = 12  !  Hex
         inquire ( file = 'quat.symm.hex' , exist=found )
         if ( .not.found ) then
            print * , 'Did not find the symmetry file, quat.symm.hex'
            call exit ( 1 )
         end if
      elseif(iq_symm.eq.3) then
         n_max = 4   !  Orth
         inquire ( file = 'quat.symm.ort' , exist=found )
         if ( .not.found ) then
            print * , 'Did not find the symmetry file, quat.symm.ort'
            call exit ( 1 )
         end if
      endif
!     print*, 'reading symmetry operators from "quat.symm.cubic"'
      if ( iq_symm.eq.1 ) then
         open( unit=3,file = 'quat.symm.cubic',status='old' )
      elseif( iq_symm.eq.2 ) then
         open( unit=3,file = 'quat.symm.hex',status='old' )
      elseif( iq_symm.eq.3 ) then
         open( unit=3,file = 'quat.symm.orth',status='old' )
      endif

      read(3,*)			! skip over title line
      read(3,*) numsymm		! read number of symmetry operators
      if(numsymm.gt.n_max) then
	 numsymm = n_max
	 print*, 'too many symmetry operators, cutting back to ',n_max
      else
!     print*, 'reading ',numsymm,' symmetry operators'
      endif

      do 1800, i=1,numsymm
	 read(3,*) (quatsymm(ijk,i),ijk=1,4)
!     print*, 'symm. no. ',i,(quatsymm(ijk,i),ijk=1,4)
 1800 continue
      close(unit=3)

!     print*, 'reading sample symmetry operators from "quat.symm.ort"'
!  which contains:
!  quat.symm.ort	file with 4 proper rotations in quaternion form
!  4
!   0    0    0    1
!   1    0    0    0
!   0    1    0    0
!   0    0    1    0

!      open(unit=3,file = 'quat.symm.ort',status='old')
!      read(3,*)			! skip over title line
!      read(3,*) numsamp		! read number of sample symmetry operators
!      if(numsamp.gt.4) then
!	 numsamp=4
!	 print*, 'too many symmetry operators, cutting back to 4!'
!      else
!     print*, 'reading ',numsamp,' symmetry operators'
!      endif
!      do 1810, i=1,numsamp
!	 read(3,*) (quatsamp(ijk,i),ijk=1,4)
!     print*, 'samp. symm. no. ',i,(quatsamp(ijk,i),ijk=1,4)
! 1810 continue
!      close(unit=3)

!  now we hard-wire the four operators into the code
!  so that choices by user are easier to deal with
      numsamp = 4
      do i = 1,numsamp
	 do ijk = 1,4
	    quatsamp(ijk,i) = 0.
	 enddo
      enddo
      quatsamp(4,1) = 1.
      quatsamp(1,2) = 1.
      quatsamp(2,3) = 1.
      quatsamp(3,4) = 1.
      
      return
      end
!
! ________________________________________
!

	subroutine textur5( pointx , only_one,l_NoWeight , number2Read )

        implicit none

!  fifth version, for use in REXGBS, WTS2POP
!  only reads in orientations; symm opers in textur4 above
!  not for use in REX2D

!  following code taken from LApp68.f for reading in TEXIN

        integer pointx,ng,i,j
	real evm,fk1(9),nstate
	character*80 textitl
	character hkl*3,seclab*5,title*8,idate*8
	character eulan*25,nomen,iper*6
	character prosa*80,prosa1*80,hh,kk,ll,lper,nomon
	character*5 label,secid
	character fname*50,fname2*50
	real ph,th,ps,dph,dth,dps,sph,cph,cth,sth,sps,cps,qr(4)
	real theta,r1,r2,r3
	integer lmn
	real d1,d2,d3
	logical only_one
        logical l_NoWeight  !  for unweighted input

	integer numcsl,nsig(50),numsymm,numsamp,ngrain
	real quatsymm(4,48),quatsamp(4,4)
	real quatcsl(4,50)
	real aquat(4,2000000),phi1(2000000),capphi(2000000)
     $       ,phi2(2000000)
	real grwt(2000000)
	common/a1/ quatsymm,numsymm,quatsamp,numsamp
	common/a2/quatcsl,numcsl,nsig
	common/a3/aquat,ngrain,phi1,capphi,phi2
	common/a4/ grwt,eulan
        logical add_90
	common/strings/ fname,hkl,seclab,title,idate,textitl,add_90

        real tmp_wt,last_phi1,last_capphi,last_phi2
        integer count
!  used to consolidate orientations
        real rad
        integer number2Read , ulimit
        logical track

!  CODE::
	rad=57.29578
        ulimit = number2Read
        if ( number2Read .eq. -1 ) ulimit = 9000000

!	print*, 'Enter the KEYWORD for this run, e.g. "cube55"'
!	read(*,'(a)') fname2
!	pointx=20
!	do 5, i=20,1,-1
!		print*, 'i,fname2(i:i) ',i,fname2(i:i)
!		if(fname2(i:i).eq.' ') pointx=i
!5	continue
!	pointx=pointx-1
!	ffile = fname2(1:pointx)
!	fname2((pointx+1):(pointx+1))=char(0)
!	print*, 'fname2,pointx,fname'
!	print*, fname2,',   ',pointx,',   ',fname
!
!	print*, 'reading orientations from fname....',fname(1:(pointx+3))
	if(only_one) then
	   open(unit=3,file = fname(1:(pointx))//'.wts',status='old')
	else
	   open(unit=3,file = fname(1:(pointx+3))//'.wts',status='old')
	endif

	read(3,7) textitl
    7 format(a)
	read(3,*)
	read(3,*)  !  don't need the strain etc.
!	read(3,70) evm,fk1,nstate
   70 format(10f7.3,i3)
 	read(3,71) eulan,iper
   71 format(a25,49x,a6)
 	nomen=eulan
	if(nomen.eq.'B') print*, 'Detected Bunge angles' 
	if(nomen.eq.'K') print*, 'Detected Kocks angles' 
! 	print 76,textitl
   76 format(/
     #' TEXIN file= '/1x,a/)

      ng = 0
      count = 0
      do 529 i = 1 , ulimit
         track = .false.
!      do 520 i=1,18
! 520  state(i,ng)=0.
!      read(3,50,err=530,end=530) d1,d2,d3,grwt(ng),(state(i,ng),i=1,nstate)
         if(l_NoWeight) then
            read(3,*,end=530,err=530) d1,d2,d3
            tmp_wt = 1.
!            grwt(ng) = 1.
         else
            read(3,*,end=530,err=530) d1 , d2 , d3 , tmp_wt
         end if
         if ( add_90 ) d1 = d1 + 90.  !  hidden option 7
         ng = ng + 1
!  debug
         if ( d3 .lt. 4. ) then
            track = .true.
            print*,'i,d1,d2,d3,tmp_wt,ng:',i,d1,d2,d3,tmp_wt,ng
         end if

!         if ( d1.ge.0. .and. d1.lt.1.e-3 ) d1 = 1.0e-3
!         if ( d1.gt.89.999 .and. d1.le.90. ) d1 = 89.999
!         if ( d1.gt.179.999 .and. d1.le.180. ) d1 = 179.999
!         if ( d1.gt.359.999 .and. d1.le.360. ) d1 = 359.999
!         if ( d2.ge.0. .and. d2.lt.1.e-3 ) d2 = 1.0e-3
!         if ( d2.gt.89.999 .and. d2.le.90. ) d2 = 89.999
!         if ( d2.gt.179.999 .and. d2.le.180. ) d2 = 179.999
!         if ( d2.gt.359.999 .and. d2.le.360. ) d2 = 359.999
!         if ( d3.ge.0. .and. d3.lt.1.e-3 ) d3 = 1.0e-3
!         if ( d3.gt.89.999 .and. d3.le.90. ) d3 = 89.999
!         if ( d3.gt.179.999 .and. d3.le.180. ) d3 = 179.999
!         if ( d3.gt.359.999 .and. d3.le.360. ) d3 = 359.999
!  avoid values on the edges of the space

         if(i.gt.1) then
            if(abs(d1 - last_phi1).lt.0.1 .and.
     &           abs(d2 - last_capphi).lt.0.1 .and.
     &           abs(d3 - last_phi2).lt.0.1 ) then
               ng = ng - 1
               count = count + 1
               grwt(ng) = grwt(ng) + tmp_wt
!$$$               print*,'SAME: i,ng,count,d1,d2,d3'
!$$$               print*,i,ng,count,d1,d2,d3,grwt(ng)
!  angles are sufficiently similar to consoliate the point
!  with the previous one
            else
               grwt(ng) = tmp_wt
               count = 1
!$$$               print*,'DIFFNT: i,ng,count,d1,d2,d3,wt'
!$$$               print*,i,ng,count,d1,d2,d3,grwt(ng)
            endif  !  if(abs(...
         else
            grwt(1) = tmp_wt
!$$$            print*,'NG=1: i,ng,count,d1,d2,d3,wt'
!$$$     &           ,i,ng,count,d1,d2,d3,grwt(ng)
         endif    !  if(ng.gt.1
         last_phi1 = d1
         last_capphi = d2
         last_phi2 = d3
!  save the values

!	print*, 'ng, angles..  ',ng,d1,d2,d3
!	if((d1.eq.999.).and.(d2.eq.999.).and.(d3.eq.999.)) goto 530
 50	 format(6f8.2,24f6.2)
!      read(3,*,end=530) d1,d2,d3,grwt(ng)
!            .: for old unformatted TEXIN files

	 if(nomen.eq.'B') then 
	    phi1(ng) = d1
	    capphi(ng) = d2
	    phi2(ng) = d3
	 elseif(nomen.eq.'K') then
	    phi1(ng) = d1 + 90.
	    if( phi1(ng) .gt. 360. ) phi1(ng) = phi1(ng) - 360.
!  corrected 24 Jul 09
	    capphi(ng) = d2
	    phi2(ng) = 90. - d3
	    if( phi2(ng) .lt. 0. ) phi2(ng) = phi2(ng) + 360.
	 elseif ( nomen .eq. 'R' ) then
	    phi1(ng) = d1 + 90.
	    if(phi1(ng).gt.360. ) phi1(ng) = phi1(ng) - 360.
!  added 11 Dec 10
	    capphi(ng) = d2
	    phi2(ng) = d3 + 90.
	    if( phi2(ng) .gt. 360.) phi2(ng) = phi2(ng) - 360.
	 else
	    stop 'cannot deal with that type! '
	 endif

	 if(nomen.eq.'B') then 
	    call quatB(d1/rad,d2/rad,d3/rad,qr)
	 elseif(nomen.eq.'K') then
	    d1 = d1 + 90.
	    if( d1 .gt. 360. ) d1 = d1 - 360.
	    d3 = 90.0 - d3
	    if ( d3 .lt. 0. ) d3 = d3 + 360.
	    call  quatB((d1/rad),d2/rad,(d3/rad),qr)
	 elseif ( nomen .eq. 'R' ) then
	    d1 = d1 + 90.
	    if ( d1 .gt. 360. ) d1 = d1 - 360.
	    d3 = d3 + 90.
	    if( d3 .gt. 360. ) d3 = d3 - 360.
	    call  quatB((d1/rad),(d2/rad),(d3/rad),qr)
	 else
	    stop 'cannot deal with that type! '
	 end if
         if ( track ) print*,'d1,d2,d3: ' , d1 ,d2 ,d3
!  use the new conversion routine based on Altmann if Bunge
!  else, use longer routine

!         print*,'textur5: ng, quat: ',ng,qr
	 do j=1,4
	    aquat(j,ng)=qr(j)
	 enddo

 529	continue
  530 	continue
!        print*,'end ... ng: ',ng
!	ngrain = ng-1
	ngrain = ng

  	return
	end

!
! ________________________________________
!

       subroutine texture_any(filename,only_one,l_NoWeight
     $     , number2Read )

        implicit none

!  fifth version, for use in REXGBS, WTS2POP
!  only reads in orientations; symm opers in textur4 above
!  not for use in REX2D

!  following code taken from LApp68.f for reading in TEXIN
!  same as regular WTS version, but arb. extension

        integer pointx,ng,i,j
	real evm,fk1(9),nstate
	character*80 textitl
	character hkl*3,seclab*5,title*8,idate*8
	character eulan*25,nomen,iper*6
	character prosa*80,prosa1*80,hh,kk,ll,lper,nomon
	character*5 label,secid
	character fname*50,fname2*50
	real ph,th,ps,dph,dth,dps,sph,cph,cth,sth,sps,cps,qr(4)
	real theta,r1,r2,r3
	integer lmn
	real d1,d2,d3
	logical only_one
        logical l_NoWeight  !  for unweighted input

	integer numcsl,nsig(50),numsymm,numsamp,ngrain
	real quatsymm(4,48),quatsamp(4,4)
	real quatcsl(4,50)
	real aquat(4,2000000),phi1(2000000),capphi(2000000)
     $       ,phi2(2000000)
	real grwt(2000000)
	common/a1/ quatsymm,numsymm,quatsamp,numsamp
	common/a2/quatcsl,numcsl,nsig
	common/a3/aquat,ngrain,phi1,capphi,phi2
	common/a4/ grwt,eulan
        logical add_90
	common/strings/ fname,hkl,seclab,title,idate,textitl,add_90

        real tmp_wt,last_phi1,last_capphi,last_phi2
        integer count
!  used to consolidate orientations
        real rad
	parameter ( rad = 57.29578 )
        character*50 filename  !  must match fname2
        integer number2Read , ulimit

!  CODE::
        ulimit = number2Read
        if ( number2Read .eq. -1 ) ulimit = 9000000

	if(only_one) then
	   open( unit=3 , file =  filename , status='old' )
	else
           print*,'Cannot use arbitrary extension with multiple inputs'
           stop
!	   open(unit=3,file = fname(1:(pointx+3))//'.wts',status='old')
	endif

	read(3,7) textitl
    7 format(a)
	read(3,*)
	read(3,*)  !  don't need the strain etc.
!	read(3,70) evm,fk1,nstate
   70 format(10f7.3,i3)
 	read(3,71) eulan,iper
   71 format(a25,49x,a6)
 	nomen=eulan
	if(nomen.eq.'B') print*, 'Detected Bunge angles' 
	if(nomen.eq.'K') print*, 'Detected Kocks angles' 
! 	print 76,textitl
   76 format(/
     #' TEXIN file= '/1x,a/)

      ng = 0
      count = 0
      do 529 i = 1 , ulimit
         if ( mod(i,10000).eq.0 ) print*,'reading line ',i
         if(l_NoWeight) then
            read(3,*,end=530,err=530) d1,d2,d3
            tmp_wt = 1.
!            grwt(ng) = 1.
         else
            read(3,*,end=530,err=530) d1,d2,d3,tmp_wt
         endif
         if ( add_90 ) d1 = d1 + 90.  !  hidden option 7
         ng = ng + 1
!$$$         print*,'i,d1,d2,d3,tmp_wt,ng:',i,d1,d2,d3,tmp_wt,ng

!         if ( d1.ge.0. .and. d1.lt.1.e-3 ) d1 = 1.0e-3
!         if ( d1.gt.89.999 .and. d1.le.90. ) d1 = 89.999
!         if ( d1.gt.179.999 .and. d1.le.180. ) d1 = 179.999
!         if ( d1.gt.359.999 .and. d1.le.360. ) d1 = 359.999
!         if ( d2.ge.0. .and. d2.lt.1.e-3 ) d2 = 1.0e-3
!         if ( d2.gt.89.999 .and. d2.le.90. ) d2 = 89.999
!         if ( d2.gt.179.999 .and. d2.le.180. ) d2 = 179.999
!         if ( d2.gt.359.999 .and. d2.le.360. ) d2 = 359.999
!         if ( d3.ge.0. .and. d3.lt.1.e-3 ) d3 = 1.0e-3
!         if ( d3.gt.89.999 .and. d3.le.90. ) d3 = 89.999
!         if ( d3.gt.179.999 .and. d3.le.180. ) d3 = 179.999
!         if ( d3.gt.359.999 .and. d3.le.360. ) d3 = 359.999
!  avoid values on the edges of the space

         if(i.gt.1) then
            if(abs(d1 - last_phi1).lt.0.1 .and.
     &           abs(d2 - last_capphi).lt.0.1 .and.
     &           abs(d3 - last_phi2).lt.0.1 ) then
               ng = ng - 1
               count = count + 1
               grwt(ng) = grwt(ng) + tmp_wt
!$$$               print*,'SAME: i,ng,count,d1,d2,d3'
!$$$               print*,i,ng,count,d1,d2,d3,grwt(ng)
!  angles are sufficiently similar to consolidate the point
!  with the previous one
            else
               grwt(ng) = tmp_wt
               count = 1
!$$$               print*,'DIFFNT: i,ng,count,d1,d2,d3,wt'
!$$$               print*,i,ng,count,d1,d2,d3,grwt(ng)
            endif  !  if(abs(...
         else
            grwt(1) = tmp_wt
!$$$            print*,'NG=1: i,ng,count,d1,d2,d3,wt'
!$$$     &           ,i,ng,count,d1,d2,d3,grwt(ng)
         endif    !  if(ng.gt.1
         last_phi1 = d1
         last_capphi = d2
         last_phi2 = d3
!  save the values

!	print*, 'ng, angles..  ',ng,d1,d2,d3
!	if((d1.eq.999.).and.(d2.eq.999.).and.(d3.eq.999.)) goto 530
 50	 format(6f8.2,24f6.2)
!      read(3,*,end=530) d1,d2,d3,grwt(ng)
!            .: for old unformatted TEXIN files

	 if(nomen.eq.'B') then 
	    phi1(ng)=d1
	    capphi(ng)=d2
	    phi2(ng)=d3
	 elseif(nomen.eq.'K') then
	    phi1(ng) = d1 + 90.
	    if(phi1(ng).lt.0.) phi1(ng) = phi1(ng) - 360.
	    capphi(ng) = d2
	    phi2(ng) = 90. - d3
	    if(phi2(ng).lt.0.) phi2(ng)=phi2(ng)+360.
	 elseif ( nomen .eq. 'R' ) then
	    phi1(ng) = d1 + 90.
	    if(phi1(ng).gt.360. ) phi1(ng) = phi1(ng) - 360.
!  added 11 Dec 10
	    capphi(ng) = d2
	    phi2(ng) = d3 + 90.
	    if( phi2(ng) .gt. 360.) phi2(ng) = phi2(ng) - 360.
	 endif
 20	 dth=d2
	 dps=d1
	 if(nomen.eq.'B') then 
	    call quatB(d1/rad,d2/rad,d3/rad,qr)
	 elseif(nomen.eq.'K') then
	    d1=d1+90.
	    if(d1.gt.360.) d1=d1-360.
	    d3=90.0-d3
	    if(d3.lt.0.) d3=d3+360.
	    call  quatB((d1/rad),d2/rad,(d3/rad),qr)
	 elseif ( nomen .eq. 'R' ) then
	    d1 = d1 + 90.
	    if ( d1 .gt. 360. ) d1 = d1 - 360.
	    d3 = d3 + 90.
	    if( d3 .gt. 360. ) d3 = d3 - 360.
	    call  quatB((d1/rad),(d2/rad),(d3/rad),qr)
	 else
	    stop 'cannot deal with that type! '
	 endif
!  use the new conversion routine based on Altmann if Bunge
!  else, use longer routine

!         print*,'textur5: ng, quat: ',ng,qr
	 do j=1,4
	    aquat(j,ng)=qr(j)
	 enddo

 529	continue
  530 	continue
!        print*,'end ... ng: ',ng
!	ngrain = ng-1
	ngrain = ng

  	return
	end

! ________________________________________

      subroutine texture_GB( pointx , only_one
     $     , l_NoWeight , number2Read , read_GB
     $     , read_GB_degr )
!  reads grain boundary segment lists, v 11
      implicit none

      integer pointx,ng,i,j
      character*80 textitl
      character hkl*3,seclab*5,title*8,idate*8
      character eulan*25 , nomen , iper*6
      character fname*50 , fname2*50
      real ph,th,ps,dph,dth,dps,sph,cph,cth,sth,sps,cps,qr(4)
      real theta,r1,r2,r3, x1, x2, y1, y2
      integer lmn
      real d1 , d2 , d3 , d4 , d5 , d6
      logical only_one
      logical l_NoWeight        !  for unweighted input

      integer numcsl,nsig(50),numsymm,numsamp,ngrain
      real quatsymm(4,48),quatsamp(4,4)
      real quatcsl(4,50)
      real aquat(4,2000000),phi1(2000000),capphi(2000000),phi2(2000000)
      real grwt(2000000)
      common/a1/ quatsymm,numsymm,quatsamp,numsamp
      common/a2/quatcsl,numcsl,nsig
      common/a3/aquat,ngrain,phi1,capphi,phi2
      common/a4/ grwt , eulan
      logical add_90
      common/strings/ fname,hkl,seclab,title,idate,textitl,add_90

      real tmp_wt , last_phi1 , last_capphi , last_phi2
      real last_phi1B , last_capphiB , last_phi2B
      integer count
!  used to consolidate orientations
      real rad
      logical last_header
      character inline*132
      integer ind
      real dum1,dum2,dum3,dum4,dum5
      integer number2Read , ulimit
      logical read_GB , read_GB_degr

!  CODE::
      print * , 'starting texture_GB with fname= ' , fname
      rad = 57.29578
      ulimit = number2Read
      if ( number2Read .eq. -1 ) ulimit = 9000000

      if( only_one ) then
         open( unit=3, file = fname(1:(pointx))//'.txt',status='old')
      else
         open( unit = 3 , file = fname(1:(pointx+3))//'.txt'
     $        , status = 'old' )
      end if

      count = 0
      last_header = .false.
      do while( .not.last_header )
         read( 3 , "(a)" ) inline
         print"('inline:',132a)",inline
         ind = 0
         ind = index( inline , "#" )  
!  allows for comment lines (although none in my examples)
!         print*,'ind = ',ind, ' last_header ',last_header
         if ( ind .eq. 1 ) then
            last_header = .false.
            count = count + 1
         else
            last_header = .true. !  make it continue unless # found
         end if
      end do
      print*,'texture_GB: read ',count,' header lines'

      rewind ( 3 )
      if ( count .gt. 0 ) then
         do i = 1 , count
            read(3,*)
         end do
      end if

!      print*,'texture_GB: reading data lines'
!      print*, 'Bunge Euler angles assumed' 
      ng = -1  !  special case for 2 per line
      count = 0
      do 529 i = 1 , ulimit
         if ( mod(i,500000).eq.0 ) print*,'texture_GB: reading line ',i
         if(l_NoWeight) then
!            read(3,*,end=530,err=530) dum1,dum2,dum3,dum4,dum5,d1,d2,d3
            read( 3 , * , end=530 ) d1 , d2 , d3 , d4 , d5 , d6
            tmp_wt = 1.
         else
            read( 3 , * , end=530 ) d1 , d2 , d3 , d4 , d5 , d6
     $           , dum1, dum2, x1, y1, x2, y2   !read coors, calc length, assign to tmp wt
	               tmp_wt=sqrt(((x2-x1)**2)+((y2-y1)**2))
         end if
                                               
         ng = ng + 2
!         print*,'read_GB: ng, angles: ', ng , d1 , d2 , d3,d4,d5,d6
! debug
         if ( read_GB ) then
            d1 = d1 * rad
            d2 = d2 * rad
            d3 = d3 * rad
            d4 = d4 * rad
            d5 = d5 * rad
            d6 = d6 * rad
         end if
         if ( add_90 ) then
            d1 = d1 + 90.       !  hidden option 7
         end if

!  now we check to see if we have a series of similar Eulers
         if ( i.gt.1 ) then
            if(  abs(d1 - last_phi1).lt.0.1 .and.
     &           abs(d2 - last_capphi).lt.0.1 .and.
     &           abs(d3 - last_phi2).lt.0.1 ) then
!  similar enough to assume it's the same and accumulate the weight
               ng = ng - 2
               count = count + 1
               grwt(ng) = grwt(ng) + tmp_wt
               grwt(ng+1) = grwt(ng+1) + tmp_wt
!$$$               print*,'SAME: i,ng,count,d1,d2,d3'
!$$$               print*,i,ng,count,d1,d2,d3,grwt(ng)
!  angles are sufficiently similar to consolidate the point
!  with the previous one
            else  !  or it's different 
               grwt(ng) = tmp_wt
               grwt(ng+1) = tmp_wt
               count = 1
!$$$               print*,'DIFFNT: i,ng,count,d1,d2,d3,wt'
!$$$               print*,i,ng,count,d1,d2,d3,grwt(ng)
            endif  !  if(abs(...
         else
            grwt(1) = tmp_wt
            grwt(2) = tmp_wt
!$$$            print*,'NG=1: i,ng,count,d1,d2,d3,wt'
!$$$     &           ,i,ng,count,d1,d2,d3,grwt(ng)
         end if    !  if(ng.gt.1
         last_phi1 = d1
         last_capphi = d2
         last_phi2 = d3
!  save the values

         phi1(ng) = d1
         capphi(ng) = d2
         phi2(ng) = d3
         call quatB( (d1/rad) , (d2/rad) , (d3/rad) , qr)
	 do j = 1 , 4
	    aquat(j,ng) = qr(j)
	 end do
c$$$         print*,'texture_GB: i,ng,d1,d2,d3,quat,wt'
c$$$     &        ,i,ng,d1,d2,d3,qr,grwt(ng)
!  debug

!  now for the 2nd orientation
         if ( add_90 ) then
            d4 = d4 + 90.       !  hidden option 7
         end if

         phi1(ng+1) = d4
         capphi(ng+1) = d5
         phi2(ng+1) = d6
         call quatB( (d4/rad) , (d5/rad) , (d6/rad) , qr)
	 do j = 1 , 4
	    aquat( j , ng+1 ) = qr(j)
	 end do

 529  end do
 530  continue
!	ngrain = ng-1
	ngrain = ng + 1
        print*,'TEXTURE_GB: ' , ngrain , ' grains read in'

  	return
	end

!
! ________________________________________
!

      subroutine texture_wquat( pointx ,only_one , l_NoWeight
     $     , number2Read )

      implicit none

!  fifth version, for use in REXGBS, WTS2POP
!  only reads in orientations; symm opers in textur4 above
!  not for use in REX2D

!  following code taken from LApp68.f for reading in TEXIN

!  based on textur5

      integer pointx,ng,i,j
      real evm,fk1(9),nstate
      character*80 textitl
      character hkl*3,seclab*5,title*8,idate*8
      character eulan*25,nomen,iper*6
      character prosa*80,prosa1*80,hh,kk,ll,lper,nomon
      character*5 label,secid
      character fname*50,fname2*50
      real ph,th,ps,dph,dth,dps,sph,cph,cth,sth,sps,cps,qr(4)
      real theta,r1,r2,r3
      integer lmn
      real d1,d2,d3
      logical only_one
      logical l_NoWeight        !  for unweighted input

      integer numcsl,nsig(50),numsymm,numsamp,ngrain
      real quatsymm(4,48),quatsamp(4,4)
      real quatcsl(4,50)
      real aquat(4,2000000),phi1(2000000),capphi(2000000),phi2(2000000)
      real grwt(2000000)
      common/a1/ quatsymm,numsymm,quatsamp,numsamp
      common/a2/quatcsl,numcsl,nsig
      common/a3/aquat,ngrain,phi1,capphi,phi2
      common/a4/ grwt,eulan
      logical add_90
      common/strings/ fname,hkl,seclab,title,idate,textitl,add_90

      real tmp_wt,last_phi1,last_capphi,last_phi2
      integer count
!  used to consolidate orientations
      real rad
      logical last_header
      character inline*132
      integer ind
      real dum1,dum2,dum3,dum4,dum5
      integer number2Read , ulimit 

!  CODE::
      rad = 57.29578
      ulimit = number2Read
      if ( number2Read .eq. -1 ) ulimit = 9000000

      if(only_one) then
         open(unit=3, file = fname(1:(pointx))//'.wquat',status='old')
      else
         open( unit = 3 , file = fname(1:(pointx+3))//'.wquat'
     $        , status = 'old' )
      endif

      count = 0
      last_header = .false.
      do while( .not.last_header )
         read(3,"(a)") inline
!         print"('inline:',132a)",inline
         ind = 0
         ind = index( inline , "#" )  
!  allows for comment lines (although none in my examples)
!         print*,'ind = ',ind, ' last_header ',last_header
         if ( ind .eq. 1 ) then
            last_header = .false.
            count = count + 1
         else
            last_header = .true. !  make it continue unless # found
         endif
      enddo
      print*,'read_wquat: read ',count,' header lines'

      rewind ( 3 )
      if ( count .gt. 0 ) then
         do i = 1 , count
            read(3,*)
         enddo
      endif

!      print*,'read_wquat: reading data lines'
      nomen = 'B'
!      print*, 'Bunge Euler angles assumed' 
      ng = 0
      count = 0
      do 529 i = 1 , ulimit
         if(l_NoWeight) then
!            read(3,*,end=530,err=530) dum1,dum2,dum3,dum4,dum5,d1,d2,d3
            read( 3 , * , end=530 ) dum1,dum2,dum3,dum4,dum5,d1,d2,d3
 313        format(10f8.3)
            tmp_wt = 1.
         else
!            read(3,*,end=530,err=530)
            read( 3 , * , end=530 )
     $           qr(4) , qr(1) , qr(2) , qr(3) , tmp_wt
         endif
!$$$         do j = 1 , 3
!$$$            qr(j) = -1. * qr(j) ! active rotation???
!$$$         enddo
         ng = ng + 1
!         print*,'i,d1,d2,d3,tmp_wt,ng:',i,d1,d2,d3,tmp_wt,ng
         grwt(ng) = tmp_wt

	 call q2eulB( d1 , d2 , d3 , qr )
!         print*,'textur_wquat: ng, angles: ', ng , d1 , d2 , d3
! debug
         d1 = d1 * rad
         d2 = d2 * rad
         d3 = d3 * rad
         if ( add_90 ) then
            d1 = d1 + 90.       !  hidden option 7
	    call  quatB((d1/rad),(d2/rad),(d3/rad),qr)
         end if

!         if ( d1.ge.0. .and. d1.lt.1.e-3 ) d1 = 1.0e-3
!         if ( d1.gt.89.999 .and. d1.le.90. ) d1 = 89.999
!         if ( d1.gt.179.999 .and. d1.le.180. ) d1 = 179.999
!         if ( d1.gt.359.999 .and. d1.le.360. ) d1 = 359.999
!         if ( d2.ge.0. .and. d2.lt.1.e-3 ) d2 = 1.0e-3
!         if ( d2.gt.89.999 .and. d2.le.90. ) d2 = 89.999
!         if ( d2.gt.179.999 .and. d2.le.180. ) d2 = 179.999
!         if ( d2.gt.359.999 .and. d2.le.360. ) d2 = 359.999
!         if ( d3.ge.0. .and. d3.lt.1.e-3 ) d3 = 1.0e-3
!         if ( d3.gt.89.999 .and. d3.le.90. ) d3 = 89.999
!         if ( d3.gt.179.999 .and. d3.le.180. ) d3 = 179.999
!         if ( d3.gt.359.999 .and. d3.le.360. ) d3 = 359.999
!  avoid values on the edges of the space

         phi1(ng) = d1
         capphi(ng) = d2
         phi2(ng) = d3
	 do j = 1 , 4
	    aquat(j,ng) = qr(j)
	 enddo

 529	continue
  530 	continue
!        print*,'TEXTURE_WQUAT: ' , ng , ' grains read in'
!	ngrain = ng-1
	ngrain = ng

  	return
	end

!
! ________________________________________
!

      subroutine texture_ctf(pointx,only_one,l_NoWeight , number2Read )

      implicit none

!  fifth version, for use in REXGBS, WTS2POP
!  only reads in orientations; symm opers in textur4 above
!  not for use in REX2D

!  following code taken from LApp68.f for reading in TEXIN

!  based on textur5

      integer pointx,ng,i,j
      real evm,fk1(9),nstate
      character*80 textitl
      character hkl*3,seclab*5,title*8,idate*8
      character eulan*25,nomen,iper*6
      character prosa*80,prosa1*80,hh,kk,ll,lper,nomon
      character*5 label,secid
      character fname*50,fname2*50
      real ph,th,ps,dph,dth,dps,sph,cph,cth,sth,sps,cps,qr(4)
      real theta,r1,r2,r3
      integer lmn
      real d1,d2,d3
      logical only_one
      logical l_NoWeight        !  for unweighted input

      integer numcsl,nsig(50),numsymm,numsamp,ngrain
      real quatsymm(4,48),quatsamp(4,4)
      real quatcsl(4,50)
      real aquat(4,2000000),phi1(2000000),capphi(2000000),phi2(2000000)
      real grwt(2000000)
      common/a1/ quatsymm,numsymm,quatsamp,numsamp
      common/a2/quatcsl,numcsl,nsig
      common/a3/aquat,ngrain,phi1,capphi,phi2
      common/a4/ grwt,eulan
      logical add_90
      common/strings/ fname,hkl,seclab,title,idate,textitl,add_90

      real tmp_wt,last_phi1,last_capphi,last_phi2
      integer count
!  used to consolidate orientations
      real rad
      logical last_header
      character inline*132
      integer ind
      real dum1,dum2,dum3,dum4,dum5
      integer number2Read , ulimit

!  CODE::
      rad=57.29578
      ulimit = number2Read
      if ( number2Read .eq. -1 ) ulimit = 9000000

      if(only_one) then
         open(unit=3,file = fname(1:(pointx))//'.ctf',status='old')
      else
         open(unit=3,file = fname(1:(pointx+3))//'.ctf',status='old')
      endif

      count = 0
      last_header = .true.
      do while(last_header)
         read(3,"(a)") inline
!         print"('inline:',132a)",inline
         count = count + 1
         ind = 0
         ind = index(inline,"Euler1")
!         print*,'ind = ',ind
         if ( ind .ne. 0 ) last_header = .false.
      enddo
      print*,'read_ctf: read ',count,' header lines'

      print*,'read_ctf: reading data lines'
      nomen = 'B'
      print*, 'Bunge Euler angles in degrees assumed' 
      ng = 0
      count = 0
      do 529 i = 1 , ulimit
         if(l_NoWeight) then
!            read(3,*,end=530,err=530) dum1,dum2,dum3,dum4,dum5,d1,d2,d3
            read( 3 , * , end=530 ) dum1,dum2,dum3,dum4,dum5,d1,d2,d3
 313        format(10f8.3)
            tmp_wt = 1.
         else
!            read(3,*,end=530,err=530)
            read( 3 , * , end=530 )
     $           dum1,dum2,dum3,dum4,dum5,d1,d2,d3,tmp_wt
         endif
         if ( add_90 ) then
            d1 = d1 + 90.       !  hidden option 7
         end if
         ng = ng + 1
!         print*,'i,d1,d2,d3,tmp_wt,ng:',i,d1,d2,d3,tmp_wt,ng

!         if ( d1.ge.0. .and. d1.lt.1.e-3 ) d1 = 1.0e-3
!         if ( d1.gt.89.999 .and. d1.le.90. ) d1 = 89.999
!         if ( d1.gt.179.999 .and. d1.le.180. ) d1 = 179.999
!         if ( d1.gt.359.999 .and. d1.le.360. ) d1 = 359.999
!         if ( d2.ge.0. .and. d2.lt.1.e-3 ) d2 = 1.0e-3
!         if ( d2.gt.89.999 .and. d2.le.90. ) d2 = 89.999
!         if ( d2.gt.179.999 .and. d2.le.180. ) d2 = 179.999
!         if ( d2.gt.359.999 .and. d2.le.360. ) d2 = 359.999
!         if ( d3.ge.0. .and. d3.lt.1.e-3 ) d3 = 1.0e-3
!         if ( d3.gt.89.999 .and. d3.le.90. ) d3 = 89.999
!         if ( d3.gt.179.999 .and. d3.le.180. ) d3 = 179.999
!         if ( d3.gt.359.999 .and. d3.le.360. ) d3 = 359.999
!  avoid values on the edges of the space

         if(i.gt.1) then
            if(abs(d1 - last_phi1).lt.0.1 .and.
     &           abs(d2 - last_capphi).lt.0.1 .and.
     &           abs(d3 - last_phi2).lt.0.1 ) then
               ng = ng - 1
               count = count + 1
               grwt(ng) = grwt(ng) + tmp_wt
!$$$               print*,'SAME: i,ng,count,d1,d2,d3'
!$$$               print*,i,ng,count,d1,d2,d3,grwt(ng)
!  angles are sufficiently similar to consoliate the point
!  with the previous one
            else
               grwt(ng) = tmp_wt
               count = 1
!$$$               print*,'DIFFNT: i,ng,count,d1,d2,d3,wt'
!$$$               print*,i,ng,count,d1,d2,d3,grwt(ng)
            endif  !  if(abs(...
         else
            grwt(1) = tmp_wt
!$$$            print*,'NG=1: i,ng,count,d1,d2,d3,wt'
!$$$     &           ,i,ng,count,d1,d2,d3,grwt(ng)
         endif    !  if(ng.gt.1
         last_phi1 = d1
         last_capphi = d2
         last_phi2 = d3
!  save the values

!	print*, 'ng, angles..  ',ng,d1,d2,d3
!	if((d1.eq.999.).and.(d2.eq.999.).and.(d3.eq.999.)) goto 530
 50	 format(6f8.2,24f6.2)
!      read(3,*,end=530) d1,d2,d3,grwt(ng)
!            .: for old unformatted TEXIN files

         phi1(ng) = d1
         capphi(ng) = d2
         phi2(ng) = d3
 20	 dth = d2
	 dps = d1
         call quatB(d1/rad,d2/rad,d3/rad,qr)
!  use the new conversion routine based on Altmann if Bunge
!  else, use longer routine

!         print*,'textur5: ng, quat: ',ng,qr
	 do j = 1 , 4
	    aquat(j,ng) = qr(j)
	 end do

 529	end do
  530 	continue
!        print*,'end ... ng: ',ng
!	ngrain = ng-1
	ngrain = ng

  	return
	end

!
!  ------------------
!
	subroutine readsod(pointx,only_one)
!  taken from SOD2VF_17JUN05.F

	implicit none

	integer nval,pointx
	character*80 textitl
	character prosa*80,prosa1*80,hh,kk,ll,lper
	character eulan*25,nomen*5
	character*5 label,secid
	character fname*50,one,ten,fname2*50,tname*6
	character hkl*3,seclab*5,title*8,idate*8
	real delth,rm,delom,pm
	integer iw,jw,iper(3),iavg,ngr
	integer llimit,ulimit,ik,nquads,numsamp
	logical rescale
	character afile*20, file*19, out1*22
	integer nvint(73,20,73),nscale
	integer ntype,i,j,k,ii,jj,kl,angle_type,nvmx
	real scalef
!	real sintens(73,19,19)   ! store the intensities from the SOD here
	integer num_sections
	logical only_one
	real rad
        real oldcapphi,newcapphi,darea(20)
	character header*100

	integer ngrain,ng
	real d1,d2,d3,dth,dps,qr(4)
	real aquat(4,2000000),phi1(2000000)
     &	,capphi(2000000),phi2(2000000)
	real grwt(2000000)
	common/a3/aquat,ngrain,phi1
     &	,capphi,phi2
	common/a4/ grwt,eulan
        logical add_90
	common/strings/ fname,hkl,seclab,title,idate,textitl,add_90

!  CODE::
 1011	format(a)

	rad = 57.29578
	print*,' '
        print*,'In Read_SOD'
	oldcapphi = 0.
	do 100, i = 1 , 19
           newcapphi = (float(i-1)*5.0)+2.5
           newcapphi = amax1( newcapphi , 0.0 )
           newcapphi = amin1( newcapphi , 90.0 )
           darea(i) = cos(oldcapphi/rad) - cos(newcapphi/rad)
           oldcapphi = newcapphi
100	end do
!	print*, 'completed 100 loop; DAREA()= ',darea

	if(only_one) then
	   open(unit=11,file = fname(1:(pointx))//'.sod',status='old')
	else
	   open(unit=11,file = fname(1:(pointx+3))//'.sod',status='old')
	endif

	ntype = 1
        open(unit=3,file=fname(1:(pointx+3))//'_Z.wts',status='unknown')
        print*,'Find a list of weighted orientations in _Z.wts'
        write( 3 , "(a,a)" ) ' WTS file from wts2pop, from ',fname
        header='Evm    F11    F12    F13    F21    '
     &       //'F22    F23    F31    F32    F33 nstate'
        write( 3 , "(a)" ) header 
 70     format(10f7.3,i3)
        write(3,70) 0.,1.,0.,0., 0.,1.,0., 0.,0.,1. , 1
 71     format(a5,54x,a6)
        write(3,71) 'Bunge',' 1 2 3'

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!	read in the input data
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!     read the first header lines

	do 725, k = 1 , 73

	   read(11,"(a)",err=932,end=932) textitl
	   if ( k.eq.1 ) print * , '1st line:  title=',textitl
	   read(11,1332) nomen,delth,rm,delom,pm,iw,jw
     1        ,(iper(i),i=1,3),iavg,ngr,seclab,label
 1332	   format(a5,4f5.1,5i2,2i5,2a5)

	   if(k.eq.1) then
	      nquads = int((pm+0.01)/90.)	!  +.01 to be certain
!	print*, 'debug: number of lines per PHI value (nquads)= ',nquads

	      if( ntype.eq.2 ) then
!  now check the range of phi2 for COD files
		 if( pm.eq.90 .and. numsamp.ne.4 ) then
		    print*, 'Check sample symmetry!'
		    print*, 'Range of azimuth = ',pm
		    print*, 'but no. of sample symmetry operators= '
     $                   ,numsamp
		 endif
		 if(pm.eq.180.and.numsamp.ne.2) then
		    print*, 'Check sample symmetry!'
		    print*, 'Range of azimuth = ',pm
		    print*, 'but no. of sample symmetry operators= '
     $                   ,numsamp
		 endif
		 if(pm.eq.360.and.numsamp.ne.1) then
		    print*, 'Check sample symmetry!'
		    print*, 'Range of azimuth = ',pm
		    print*, 'but no. of sample symmetry operators= '
     $                   ,numsamp
		 endif
	      endif

!  on the first section, detect the type of Euler angles in use
	      angle_type=1
	      if(nomen(5:5).eq.'B'.or.nomen(4:4).eq.'B') then
		 print*, 'detected Bunge angles'
	      endif
!  assume Bunge angles
	      if(nomen(5:5).eq.'K'.or.nomen(4:4).eq.'K') then
		 print*, 'detected Kocks angles'
		 angle_type=2
!$$$		 stop 'Can only handle Bunge angles for now!'
	      endif
!  Kocks angles
	   endif

	   scalef = 100./float(iavg)
	   rescale = (abs(scalef-1.0).gt.0.05)
	   if(rescale) print*, 'Rescaling intensities for k=',k,scalef
!  re-scale all the intensities in the section by the ratio
!  of the nominal average (100) divided by IAVG in the file
!  but require more than a 5% change to do it

	   do j = 1 , 19
	      llimit = 1
	      do ik = 1,nquads
		 ulimit = llimit+17
		 if(ik.eq.nquads) ulimit=ulimit+1
		 read(11,31,err=932) (nvint(i,j,k),i=llimit,ulimit)
 31		 format(1x,19i4)
		 if(rescale) then
		    do i=llimit,ulimit
		       nvint(i,j,k) = int(float(nvint(i,j,k))*scalef)
		    enddo
		 endif
		 llimit = llimit+18
	      end do		!  ik=
 724	   end do		!  j=

	   read(11,*,err=932,end=932)
!  blank line after each section

 725	end do
	num_sections = 73

 933	ng = 0
 	nvmx = nvint(1,1,1)
	do k = 1 , num_sections
	   do 722, j = 1 , 19
	      do 729, i = 1 , 19
		 ng = ng + 1
		 if(ntype.eq.1) then !  SOD
!		    sintens(k,j,i)=0.01*nvint(i,j,k) ! switch 1st/3rd indices
		    d1 = 5. * float(k-1)
		    d2 = 5. * float(j-1)
		    d3 = 5. * float(i-1)
		 elseif(ntype.eq.2) then !  COD
!		    sintens(i,j,k)=0.01*nvint(i,j,k) !  end result of subroutine
		    d1 = 5. * float(i-1)
		    d2 = 5. * float(j-1)
		    d3 = 5. * float(k-1)
		 endif
		 grwt(ng) = float(nvint(i,j,k)) * darea(j)
                 if ( k.eq.1 .or. k.eq.num_sections ) then
                    grwt(ng) = grwt(ng) * 0.5
                 end if  !  edge correction
c$$$                 if ( j.eq.1 .or. j.eq.19 ) then
c$$$                    grwt(ng) = grwt(ng) * 0.5
c$$$                 end if  !  edge correction
!  note the calculation of DAREA above, which already deals with PHI
                 if ( i.eq.1 .or. i.eq.19 ) then
                    grwt(ng) = grwt(ng) * 0.5
                 end if  !  edge correction
!  correction for size of area is critical!

!                 call adjust(d1)
!                 call adjust(d2)
!                 call adjust(d3)
! **********  this will need to be different for HEX input!!!!
!  anyway turned off, 24 iv 11, ADR

!		 if(nvint(i,j,k).le.0.and.nscale.eq.2) nvint(i,j,k)=1
!  reset intensity to 1 if we are using a log scale
		 if( nvint(i,j,k).gt.nvmx ) nvmx = nvint(i,j,k)

                 if ( add_90 ) then
                    d1 = d1 + 90. !  hidden option 7
                 end if
		 if( angle_type.eq.2 ) then 
		    d1 = d1 + 90.
		    d3 = 90.0 - d3
                 end if
                 if( d1.gt.360. ) d1 = d1 - 360.
                 if( d3.lt.0. )   d3 = d3 + 360.
                 phi1(ng) = d1
                 capphi(ng) = d2
                 phi2(ng) = d3
                 call quatB(d1/rad,d2/rad,d3/rad,qr)
!  use the new conversion routine based on Altmann if Bunge
!  else, use longer routine

		 do ik = 1,4
		    aquat(ik,ng) = qr(ik)
		 end do
c$$$		 print"('Bunge angles, grwt: ',4(2x,f10.2))"
c$$$     1                ,phi1(ng),capphi(ng),phi2(ng),grwt(ng)
		 write( 3, "(4(1x,f10.3))" )
     1                phi1(ng) , capphi(ng) , phi2(ng) , grwt(ng)
 729	      end do		!  i=
 722	   end do		!  j=
!           pause 'check angle values ...'
	end do			!  k=

	ngrain = ng

	write(*,492) nvmx
 492	format('READ_SOD: Maximum intensity from SOD =',i7)

        close(3)
        close(11)
	return

! 932	stop 'Input file is not in the right format.'
 932	continue
	
	if(j.eq.20) num_sections = j-1
	if(j.eq.21) num_sections = j-2	! allow for projection
	if(j.eq.38) num_sections = j-1
	if(j.eq.39) num_sections = j-2	! allow for projection
	print*,'j, num_sections,label =',j,num_sections,label
	if(num_sections.eq.19) then
	   print*,'SOD has 19 sections '
	   goto 933
	elseif(num_sections.eq.37) then
	   print*,'SOD has 37 sections '
	   goto 933
	elseif(num_sections.eq.73) then
	   print*,'SOD has 73 sections '
	   goto 933
	else
	   print*,'SOD has unusable size '
	   stop
	endif

	end

!  ________________________________

      subroutine adjust(angle)

      real angle

!  edited 8 x 06, adr

!  CODE::
      if(abs(angle).le.1.e-3) then
         angle = 1.e-3
      endif

      if(abs(angle-90.).lt.1.e-3) then
!         angle = 90.001
         angle = 89.999
      endif

      if(abs(angle-180.).lt.1.e-3) then
!         angle = 180.001
         angle = 179.999
      endif

      if(abs(angle-270.).lt.1.e-3) then
!         angle = 270.001
         angle = 269.999
      endif

      if(abs(angle-360.).lt.1.e-3) then
!         angle = 360.001
         angle = 359.999
      endif

      return
      end

! ________________________________________
!
	subroutine quatB(p1,p,p2,q)
!  convert Bunge Euler angles (radians) to quaternion; 
!  direct conversion from angles, see Altmann's book
!  the rotation is a vector transformation (active rotation)

	real p1,p,p2,q(4)
	double precision c1,c,c2,s1,s,s2

!       PI=3.14159265  !!not needed!!
!  form cosine, sine of Phi, and sum & diff of phi1, phi2
       S=DSIN(0.5d0*P)
       C=DCOS(0.5d0*P)
       S1=DSIN(0.5d0*(P1-P2))
       C1=DCOS(0.5d0*(P1-P2))
       S2=DSIN(0.5d0*(P1+P2))
       C2=Dcos(0.5d0*(P1+P2))
!      print*, 's1,c1,s,c,s2,c2'
!      print*, s1,c1
!      print*, s,c
!      print*, s2,c2
	q(1)=s*c1
	q(2)=s*s1
	q(3)=c*s2
	q(4)=c*c2
	return
	end
!
!  ______________________
!
	subroutine q2eulB(p1,p,p2,qq)

!  converts quaternion to Bunge Euler angles in radians
!  based on Altmann's solution for Euler->quat

	real qq(4),p1,p,p2
	real sum,diff
        real pi
        parameter ( pi = 3.14159265 )

!  CODE::

	if((abs(qq(2)).lt.1e-35).and.(abs(qq(1)).lt.1e-35)) then
!		diff=pi/4.
		diff = 0.  !  different choice!!!
	else
		diff = atan2(qq(2),qq(1))
	endif
!  ATAN2 is unhappy is both arguments are exactly zero!

	if((abs(qq(3)).lt.1e-35).and.(abs(qq(4)).lt.1e-35)) then
!		sum=pi/4.
		sum = 0.  !  different choice!!!
	else
		sum = atan2(qq(3),qq(4))
	endif

	p1 = diff + sum
	p2 = sum - diff
	tmp = sqrt( qq(3)**2 + qq(4)**2 )
	if(tmp.gt.1.0) tmp = 1.0
	if(tmp.lt.-1.0) tmp = -1.0
	p = 2.*acos(tmp)

!$$$	print*, 'q2EulB: quaternion input= ',qq
!$$$	print*, 'q2EulB: Bunge angles output= ',p1,p,p2
!$$$	print*, 'q2EulB: angles [degrees]= '
!$$$     &       ,180.*p1/pi,180.*p/pi,180.*p2/pi

	return
	end
!
! _____________________________
!
!
	subroutine presamp(qq,lindex,qresult)
	real qq(4),qresult(4)
	integer lindex
!
	common/a1/ quatsymm(4,48),numsymm,quatsamp(4,4),numsamp
!       PI=3.14159265
!
!  algorithm for forming resultant quaternion
!  from applying a symmetry operator, QUATSYMM(n,lindex)
!  to the first quaternion/rotation
!
!  If the symm oper == O, and the input quaternion, QQ, is an active
!  rotation, then Qresult = (O x QQ) 
!  the "PRE" in the name refers to writing the operator before the
!  rotation/orientation in vector/tensor notation:  Q' = O x Q
!  or in conventional quaternion notation, Qresult = QQ  Qsymm
!
!  Thus, for active rotations (standard definition of orientation in
!  mechanics) this is suitable for applying SAMPLE symmetry
!
	if(lindex.gt.numsamp) stop 'error in PRESAMP, lindex>numsamp'
	if(lindex.lt.1) stop 'error in presamp, lindex<1'
	qresult(1)=qq(1)*quatsamp(4,lindex)+qq(4)*quatsamp(1,lindex)
     &	-qq(2)*quatsamp(3,lindex)+qq(3)*quatsamp(2,lindex)
	qresult(2)=qq(2)*quatsamp(4,lindex)+qq(4)*quatsamp(2,lindex)
     &	-qq(3)*quatsamp(1,lindex)+qq(1)*quatsamp(3,lindex)
	qresult(3)=qq(3)*quatsamp(4,lindex)+qq(4)*quatsamp(3,lindex)
     &	-qq(1)*quatsamp(2,lindex)+qq(2)*quatsamp(1,lindex)
	qresult(4)=qq(4)*quatsamp(4,lindex)-qq(1)*quatsamp(1,lindex)
     &	-qq(2)*quatsamp(2,lindex)-qq(3)*quatsamp(3,lindex)
!
!	print*, 'qresult ',qresult
!
	return
	end
!
! _____________________________
!
!
        subroutine postsymm(qq,lindex,qresult)
        real qq(4),qresult(4)
        integer lindex
!
!        include 'common.f'
        common/a1/ quatsymm(4,48),numsymm,quatsamp(4,4),numsamp
!       PI=3.14159265
!
!  algorithm for forming resultant quaternion/rotation
!  from applying a symmetry operator, QUATSYMM(n,lindex)
!  to the first quaternion/rotation, QQ
!
!  If the symm oper == O, and the input quaternion, QQ, is an active
!  rotation, then Qresult = (QQ x O) 
!  the "POST" in the name refers to writing the operator after the
!  rotation/orientation in vector/tensor notation:  Q' = Q x O
!  or in conventional quaternion notation, Qresult = Qsymm  QQ
!
!  Thus, for active rotations (standard definition of orientation in
!  mechanics) this is suitable for applying CRYSTAL symmetry
!
        if(lindex.gt.numsymm) stop 'error in presymm, lindex>numsymm'
        if(lindex.lt.1) stop 'error in presymm, lindex<1'
        qresult(1)=quatsymm(4,lindex)*qq(1)+quatsymm(1,lindex)*qq(4)
     &  +quatsymm(3,lindex)*qq(2)-quatsymm(2,lindex)*qq(3)
        qresult(2)=quatsymm(4,lindex)*qq(2)+quatsymm(2,lindex)*qq(4)
     &  +quatsymm(1,lindex)*qq(3)-quatsymm(3,lindex)*qq(1)
        qresult(3)=quatsymm(4,lindex)*qq(3)+quatsymm(3,lindex)*qq(4)
     &  +quatsymm(2,lindex)*qq(1)-quatsymm(1,lindex)*qq(2)
        qresult(4)=quatsymm(4,lindex)*qq(4)-quatsymm(1,lindex)*qq(1)
     &  -quatsymm(2,lindex)*qq(2)-quatsymm(3,lindex)*qq(3)
!
!       print*, 'qresult ',qresult
!
        return
	end
!
!
! _____________________________
!
!
	subroutine presymm(qq,lindex,qresult)
	real qq(4),qresult(4)
	integer lindex
!
	common/a1/ quatsymm(4,48),numsymm,quatsamp(4,4),numsamp
!       PI=3.14159265
!
!  algorithm for forming resultant quaternion/rotation
!  from applying a symmetry operator, QUATSYMM(n,lindex)
!  to the first quaternion/rotation, QQ
!
!  If the symm oper == O, and the input quaternion, QQ, is an active
!  rotation, then Qresult = (O x QQ) 
!  the "PRE" in the name refers to writing the operator before the
!  rotation/orientation in vector/tensor notation:  Q' = O x Q
!  or in conventional quaternion notation, Qresult = QQ  Qsymm
!
!  Thus, for active rotations (standard definition of orientation in
!  mechanics) this is suitable for applying SAMPLE symmetry
!  BUT, see PRESAMP elsewhere in this set of subroutines
!
	if(lindex.gt.numsymm) stop 'error in postsymm, lindex>numsymm'
	if(lindex.lt.1) stop 'error in postsymm, lindex<1'
	qresult(1)=qq(1)*quatsymm(4,lindex)+qq(4)*quatsymm(1,lindex)
     &	-qq(2)*quatsymm(3,lindex)+qq(3)*quatsymm(2,lindex)
	qresult(2)=qq(2)*quatsymm(4,lindex)+qq(4)*quatsymm(2,lindex)
     &	-qq(3)*quatsymm(1,lindex)+qq(1)*quatsymm(3,lindex)
	qresult(3)=qq(3)*quatsymm(4,lindex)+qq(4)*quatsymm(3,lindex)
     &	-qq(1)*quatsymm(2,lindex)+qq(2)*quatsymm(1,lindex)
	qresult(4)=qq(4)*quatsymm(4,lindex)-qq(1)*quatsymm(1,lindex)
     &	-qq(2)*quatsymm(2,lindex)-qq(3)*quatsymm(3,lindex)
!
!	print*, 'postsymm: qresult ',qresult
!
	return
	end
!
!
!  --------------------
!
	function scalar(a,b)
!  return SCALAR PRODUCT of A and B
	real scalar,a(3),b(3)
	scalar=0.
	do 100, i=1,3
	scalar=scalar+a(i)*b(i)
100	continue
	return
	end
!
!  -------------------
!
	subroutine donorm(a)
	real a(3),b(3),rsum,rnorm
	rsum=0.
	do 100, i=1,3
	rsum=rsum+a(i)**2
100	continue
	rnorm=sqrt(rsum)
	do 200, i=1,3
	b(i)=a(i)/rnorm
	a(i)=b(i)
200	continue
	return
	end
!
!  ______________________

	subroutine qrvec(qq,din,dout)

!  uses quaternion to rotate a vector from DIN to DOUT
!  note the minus sign in the line with the permutation tensor
!  we use this version to generate pole figures, which means that
!  we are effectively performing the reverse transformation
!  from crystal to sample axes (frames)

!  cleaned up code  vi 08, ADR
!  note that -q gives the same result as it should (represents same rotation)

	real qq(4),din(3),dout(3)
	real t1 , tmp

!  CODE::
!        print*,'qrvec: qq: ',qq
	t1 = qq(4)**2-qq(1)**2-qq(2)**2-qq(3)**2
	do i = 1,3
           dout(i) = t1*din(i)

           tmp = 0.
           do j = 1,3
              tmp = tmp + (qq(j)*din(j))
           enddo
           dout(i) = dout(i) + 2. * qq(i) * tmp

           do j = 1,3
              if(i.ne.j) then
                 do 80, ijk = 1,3
                    if(i.ne.ijk.and.j.ne.ijk) k=ijk
 80              continue
                 kl = j-i
                 if( kl .eq.  2 ) kl = -1
                 if( kl .eq. -2 ) kl =  1
!  poor man's permutation tensor!
!                 print*,'i,j,k,kl: ',i,j,k,kl
                 dout(i) = dout(i) - (2.*qq(k)*qq(4)*float(kl)*din(j))
              endif   !  if(i.ne.j) then
           enddo

        enddo

	return
	end
!
! _____________________________
!  }}}} 
	subroutine invqrvec( qq , din , dout )

!  uses quaternion to rotate a vector from DIN to DOUT; 
!  but in the inverse sense so as to give sample to crystal axes
!  derived from qrvec, above routine
!  note that -q gives the same result as it should (represents same rotation)

	real qq(4),din(3),dout(3)
	real t1

!  CODE::

	t1 = qq(4)**2-qq(1)**2-qq(2)**2-qq(3)**2
	do i = 1,3
           dout(i) = t1*din(i)

           tmp = 0.
           do j = 1,3
              tmp = tmp + (qq(j)*din(j))
           enddo
           dout(i) = dout(i) + 2. * qq(i) * tmp

           do j = 1,3
              if(i.ne.j) then
                 do 80, ijk = 1,3
                    if(i.ne.ijk.and.j.ne.ijk) k=ijk
 80              continue
                 kl = j-i
                 if( kl .eq.  2 ) kl = -1
                 if( kl .eq. -2 ) kl =  1
!  poor man's permutation tensor!
!                 print*,'i,j,k,kl: ',i,j,k,kl
                 dout(i) = dout(i) + (2.*qq(k)*qq(4)*float(kl)*din(j))
!  the ONLY difference between this and QRVEC is the + instead of - above
              endif   !  if(i.ne.j) then
           enddo

        enddo
	return
	end
!
! _____________________________
!
	subroutine misquat(qq,thetamin)
	real qq(4,2),thetamin,qresult(4),tmp(2),rquat(4)
	real disor
	real qmax,q1max,q2max
        real pi
        parameter ( pi = 3.14159265 )

!  algorithm for forming resultant quaternion
!  and determining minimum angle (degrees)
!  taken from Sutton & Baluffi
!  ASSUMES CUBIC CRYSTAL SYMMETRY
!  note that the resultant quaternion is not returned
!  because it is not in the fundamental zone
!
!  note change of signs to get inverse of second orientation
!
	qresult(1)=qq(1,1)*qq(4,2)-qq(4,1)*qq(1,2)
     &	+qq(2,1)*qq(3,2)-qq(3,1)*qq(2,2)
	qresult(2)=qq(2,1)*qq(4,2)-qq(4,1)*qq(2,2)
     &	+qq(3,1)*qq(1,2)-qq(1,1)*qq(3,2)
	qresult(3)=qq(3,1)*qq(4,2)-qq(4,1)*qq(3,2)
     &	+qq(1,1)*qq(2,2)-qq(2,1)*qq(1,2)
	qresult(4)=qq(4,1)*qq(4,2)+qq(1,1)*qq(1,2)
     &	+qq(2,1)*qq(2,2)+qq(3,1)*qq(3,2)
!
!	print*, 'qresult ',qresult
	qmax=0.
	iqindex=0
	do 10, i=1,4
	qresult(i)=abs(qresult(i))
	if(qresult(i).gt.qmax) then
		qmax=qresult(i)
		iqindex=i
	endif
10	continue
!
	q1max=0.
	iq1index=0
!  find the next highest q component
	do 20, i=1,4
	if(i.eq.iqindex) goto 20
	if(qresult(i).gt.q1max) then
		q1max=qresult(i)
		iq1index=i
	endif
20	continue
!
	disor=amax1(qmax,(qmax+q1max)/sqrt(2.),
     &	(qresult(1)+qresult(2)+qresult(3)+qresult(4))/2.)
	if(disor.gt.1.0) disor=1.0
	if(disor.lt.-1.0) disor=-1.0
	thetamin=acos(disor)*360./pi
!	print*,'thetamin ',thetamin
!
	q2max=0.
	iq2index=0
!  find the next highest q component
	do 30, i=1,4
	if(i.eq.iqindex.or.i.eq.iq1index) goto 30
	if(qresult(i).gt.q2max) then
		q2max=qresult(i)
		iq2index=i
	endif
30	continue
!
	rquat(4)=qmax
	rquat(3)=q1max
	rquat(2)=q2max
	do 40, i=1,4
		if(i.ne.iqindex.and.
     &	i.ne.iq1index.and.
     &	i.ne.iq2index)  rquat(1)=qresult(i)
40	continue
!  supply the sorted quaternion 
!  CAUTION:  note that q1<q2<q3<q4
!  whereas typical Rodrigues sorting is R1>R2>R3
!
	return
	end
!
! _____________________________
!
	subroutine comquat(qq,qresult)
	real qq(4,2),qresult(4)

!       PI=3.14159265

!  algorithm for forming resultant quaternion as Q1 x Q2^-1
!  where Q2 follows Q1, i.e. is the second rotation;
!  note change of signs to get inverse of second orientation

	qresult(1)=qq(1,1)*qq(4,2)-qq(4,1)*qq(1,2)
     &	+qq(2,1)*qq(3,2)-qq(3,1)*qq(2,2)
	qresult(2)=qq(2,1)*qq(4,2)-qq(4,1)*qq(2,2)
     &	+qq(3,1)*qq(1,2)-qq(1,1)*qq(3,2)
	qresult(3)=qq(3,1)*qq(4,2)-qq(4,1)*qq(3,2)
     &	+qq(1,1)*qq(2,2)-qq(2,1)*qq(1,2)
	qresult(4)=qq(4,1)*qq(4,2)+qq(1,1)*qq(1,2)
     &	+qq(2,1)*qq(2,2)+qq(3,1)*qq(3,2)
!
!	write(*,*) 'qresult ',qresult
!
	return
	end
! _____________________________
!
!
	subroutine disquat_hex(qq,index1,index2,qresult,index1a,index2a)

!  *****************************************   HCP !!
!  this version for HCP,  ADR  25 vi 05
!  the shape of the fundamental zone (FZ) for hcp is
!  a prismatic box, with opt and bottom faces at +/- pi/2*6 = 30 degrees
!  such that the limits on R3 are independent of R1 and R2 
!  and square prism faces at a distance 1 (+/- pi/2*4 = 45 degrees) from the center

!  algorithm for finding disorientation corresponding to 
!  input quaternion, QQ (from COMQUAT):  resultant quat. is in QRESULT
!  indices for symmetry operators are in INDEX1, INDEX2

	implicit none

	real qq(4),qqn(4),qresult(4),quint(4),quintn(4),qmax
	integer index1,index2,index1a,index2a,index3,iq4neg,i,j,k,ip,jp
	integer ijk,kjl
	real switch
	real rtmp
	real tan60,sin60,cos60,tan30,sin30,cos30
	real rodr(3)
	logical tests(20)
        real pi
        parameter ( pi = 3.14159265 )
        real eps
        parameter ( eps = 1.0e-5 )

!  CODE::

	tan60 = tan (pi*60./180.)
	sin60 = sin (pi*60./180.)
	cos60 = cos (pi*60./180.)
	tan30 = tan (pi*30./180.)
	sin30 = sin (pi*30./180.)
	cos30 = cos (pi*30./180.)

	index1=0
	index2=0
	index1a=0
	index2a=0
	qmax=0.
	ip=1
	jp=1

!	do 200, i=1,24
	do 200, i=1,12  !   HCP
	   call presymm(qq,i,quint)

!       do 190, j=1,24
	   do 190, j=1,12	!   HCP
!  loop over the two sides of the grain
	      call postsymm(quint,j,quintn)

	      do 185, kjl=1,2
		 switch=1.0
		 if(kjl.eq.2) then
		    switch=-1.0
		 endif
		 do 180, k=1,4
		    qqn(k)=quintn(k)*switch
 180		 continue
!  can take negative of Q because identity = +/-(0,0,0,1)
!
		 do  ijk=1,2
		    qresult(1)=qqn(1)
		    qresult(2)=qqn(2)
		    qresult(3)=qqn(3)
		    qresult(4)=qqn(4)
		    iq4neg=ijk
		    if(ijk.eq.2) qresult(4)=-1.*qqn(4)
!  take the inverse rotation, i.e. apply the switching symmetry

		    call q2rod(rodr,qresult)
!$$$		    print"('I/J/KJL/Rodr V: ',3i4,3(1x,g10.3))",i,j,kjl,rodr

		    do k = 1,20
		       tests(k) = .false.
		    enddo

		    if(rodr(2).ge.0.0) then
		       tests(1) = .true.
		       if(rodr(1).le.1.0) then
			  tests(2) = .true.
		       
!			  if((qresult(1)-qresult(2)).lt.1e-5) then
!			    if((qresult(2)-qresult(3)).lt.1e-5) then
!			      if((qresult(3)-qresult(4)).lt.1e-5) then
!			  if(qresult(2).le.qresult(1)) then

			  if(rodr(3).ge.0.) then
			     tests(3) = .true.
			     if(rodr(3).le.tan30) then
!  these two IFs put the point inside the FZ for R3
				tests(4) = .true.

				rtmp = rodr(1)*tan30 - rodr(2)
				if(rtmp.ge.0.) then
				   tests(5) = .true.
!  line from origin at 30 degr
				   rtmp = rodr(1)*cos30 + rodr(2)*sin30
				   if(rtmp.le.1.) then
				      tests(6) = .true.
!  1st sector
				      rtmp = rodr(1)*cos60
     $                                     + rodr(2)*sin60
				      if(rtmp.le.1.) then
					 tests(7) = .true.
!  2nd sector
					 if((qresult(4)-qmax)
     $                                        .gt.eps) then
					    tests(8) = .true.
					    qmax=qresult(4)
!  so now we test to see if we have a smaller angle
					    index1=i
					    index2=j
!       index3=0
					    index1a=0
					    if(ijk.eq.2) index1a=1
					    index2a=0
					    if(kjl.eq.2) index2a=1
!  record the result if we have found a quat in the fund. zone, and a smaller angle
					 endif
				      endif
				   endif
				endif
			     endif
			  endif
		       endif
		    endif
!$$$		    print*,'TESTS: ',(tests(k),k=1,8)
!$$$		    if(tests(8)) print*,'ALL = TRUE'
 210		 enddo
 185	      continue
 190	   continue
 200	continue

	if(index1.eq.0.or.index2.eq.0) stop 'DISQUAT failed!'

	call presymm(qq,index1,quint)

	call postsymm(quint,index2,qqn)
!		 write(*,"('DISQUAT: qqn = ',4f8.3)") qqn
	do 280, k=1,4
	   if(index2a.eq.1) then
!  if we had to take the negative of the Q, then do so
!  to the answer

	      qresult(k) = -1.0 * qqn(k)
!  can take negative of Q because identity = +/-(0,0,0,1)
	   else
	      qresult(k)=qqn(k)
	   endif
 280	continue

	if(index1a.eq.1) then
	   qresult(4) = -1. * qresult(4)
!  restore the 4th component if disorient found for ijk=1
	endif

!	write(*,*) 'DISQUAT: qmax= ',qmax
	return
	end
!
! _____________________________
!
!
	subroutine disquat_ang_hex(qq,angle)

!  *****************************************   HCP !!
!  this version for HCP,  ADR  25 vi 05
!  returning only an angle [degrees], not a quaternion

!  the shape of the fundamental zone (FZ) for hcp is
!  a prismatic box, with opt and bottom faces at +/- pi/2*6 = 30 degrees
!  such that the limits on R3 are independent of R1 and R2 
!  and square prism faces at a distance 1 (+/- pi/2*4 = 45 degrees) from the center

!  algorithm for finding disorientation corresponding to 
!  input quaternion, QQ (from COMQUAT):  resultant quat. is in QRESULT
!  indices for symmetry operators are in INDEX1, INDEX2

	implicit none

	real qq(4),qqn(4),qresult(4),quint(4),quintn(4),qmax
	integer index1,index2,index1a,index2a,index3,iq4neg,i,j,k,ip,jp
	integer ijk,kjl
	real switch
	real pi,rtmp,eps
	real tan60,sin60,cos60,tan30,sin30,cos30
	real rodr(3)
	logical tests(20)
	real angle
        parameter ( pi = 3.14159265 )
        parameter ( eps = 1.0e-5 )

!  CODE::

!	eps = -1.0e-5
!        PI=3.14159265
	tan60 = tan (pi*60./180.)
	sin60 = sin (pi*60./180.)
	cos60 = cos (pi*60./180.)
	tan30 = tan (pi*30./180.)
	sin30 = sin (pi*30./180.)
	cos30 = cos (pi*30./180.)

	index1=0
	index2=0
	index1a=0
	index2a=0
	qmax = 0.
	ip = 1
	jp = 1

!	do 200, i=1,24
	do 200, i=1,12  !   HCP
	   call presymm(qq,i,quint)

!       do 190, j=1,24
!$$$	   do 190, j=1,12	!   HCP
!$$$c  loop over the two sides of the grain
!$$$	      call postsymm(quint,j,quintn)

	      do 185, kjl=1,2
		 switch=1.0
		 if(kjl.eq.2) then
		    switch=-1.0
		 endif
		 do 180, k=1,4
!$$$		    qqn(k)=quintn(k)*switch
		    qqn(k)=quint(k)*switch
 180		 continue
!  can take negative of Q because identity = +/-(0,0,0,1)
!
		 do  ijk=1,2
		    qresult(1)=qqn(1)
		    qresult(2)=qqn(2)
		    qresult(3)=qqn(3)
		    qresult(4)=qqn(4)
		    iq4neg=ijk
		    if(ijk.eq.2) qresult(4)=-1.*qqn(4)
!  take the inverse rotation, i.e. apply the switching symmetry

!$$$		    call q2rod(rodr,qresult)
!$$$		    print"('I/J/KJL/Rodr V: ',3i4,3(1x,g10.3))",i,j,kjl,rodr
!$$$
!$$$		    do k = 1,20
!$$$		       tests(k) = .false.
!$$$		    enddo

!$$$		    if(rodr(2).ge.0.0) then
!$$$		       tests(1) = .true.
!$$$		       if(rodr(1).le.1.0) then
!$$$			  tests(2) = .true.
		       
!			  if((qresult(1)-qresult(2)).lt.1e-5) then
!			    if((qresult(2)-qresult(3)).lt.1e-5) then
!			      if((qresult(3)-qresult(4)).lt.1e-5) then
!			  if(qresult(2).le.qresult(1)) then

!$$$			     if(rodr(3).ge.0.) then
!$$$				tests(3) = .true.
!$$$				if(rodr(3).le.tan30) then
!$$$c  these two IFs put the point inside the FZ for R3
!$$$				   tests(4) = .true.
!$$$
!$$$				   rtmp = rodr(1)*tan30 - rodr(2)
!$$$				   if(rtmp.ge.0.) then
!$$$				      tests(5) = .true.
!$$$c  line from origin at 30 degr
!$$$				      rtmp = rodr(1)*cos30 + rodr(2)*sin30
!$$$				      if(rtmp.le.1.) then
!$$$					 tests(6) = .true.
!$$$c  1st sector
!$$$					 rtmp = rodr(1)*cos60 + rodr(2)*sin60
!$$$					 if(rtmp.le.1.) then
!$$$					    tests(7) = .true.
!  2nd sector
		    if((qresult(4)-qmax).gt.eps) then
!$$$			     tests(8) = .true.
		       qmax=qresult(4)
!  so now we test to see if we have a smaller angle
!$$$					    index1=i
!$$$					    index2=j
!       index3=0
!$$$					    index1a=0
!$$$					    if(ijk.eq.2) index1a=1
!$$$					    index2a=0
!$$$					    if(kjl.eq.2) index2a=1
!  record the result if we have found a quat in the fund. zone, and a smaller angle
		    endif
!$$$				      endif
!$$$				   endif
!$$$				endif
!$$$			     endif
!$$$			  endif
!$$$		       endif
!$$$		    endif
!$$$		    print*,'TESTS: ',(tests(k),k=1,8)
!$$$		    if(tests(8)) print*,'ALL = TRUE'
		 enddo
 185	      continue
!$$$ 190	   continue
 200	continue

!$$$	if(index1.eq.0.or.index2.eq.0) stop 'DISQUAT failed!'
!$$$
!$$$	call presymm(qq,index1,quint)
!$$$
!$$$	call postsymm(quint,index2,qqn)
!$$$c		 write(*,"('DISQUAT: qqn = ',4f8.3)") qqn
!$$$	do 280, k=1,4
!$$$	   if(index2a.eq.1) then
!$$$c  if we had to take the negative of the Q, then do so
!$$$c  to the answer
!$$$
!$$$	      qresult(k)=qqn(k)*-1.0
!$$$c  can take negative of Q because identity = +/-(0,0,0,1)
!$$$	   else
!$$$	      qresult(k)=qqn(k)
!$$$	   endif
!$$$ 280	continue
!$$$
!$$$	if(index1a.eq.1) then
!$$$	   qresult(4)=-1.*qresult(4)
!$$$c  restore the 4th component if disorient found for ijk=1
!$$$	endif

!	write(*,*) 'DISQUAT_ANG: qmax= ',qmax
	if(qmax.gt.1.) qmax = 1.
	angle = 360. / pi * acos(qmax)

	return
	end

!     
!     _____________________________
!     

	subroutine disquat_orth(qq,thetamin)

	real qq(4),qqn(4),qresult(4),quint(4),quintn(4),qmax
	integer index1,index2,index1a,index2a,index3,iq4neg,i,j,k,ip,jp
        real thetamin
        real pi
        parameter ( pi = 3.14159265 )
        real eps
        parameter ( eps = 1.0e-5 )

!  algorithm for finding disorientation corresponding to 
!  input quaternion, QQ (from COMQUAT):  resultant quat. is in QRESULT
!  indices for symmetry operators are in INDEX1, INDEX2

!  edited for orthorhombic crystal symmetry, Jul 07

!  CODE::
	index1=0
	index2=0
	index1a=0
	index2a=0
	qmax=0.
	ip=1
	jp=1

	do 200, i=1,4  !  ortho
           call presymm(qq,i,quint)

           do 190, j=1,4        !  ortho
!  loop over the two sides of the grain
              call postsymm(quint,j,quintn)

              do 185, kjl=1,2
                 switch=1.0
                 if(kjl.eq.2) then
                    switch=-1.0
                 endif
                 do 180, k=1,4
                    qqn(k)=quintn(k)*switch
 180             continue
!  can take negative of Q because identity = +/-(0,0,0,1)

                 do 210, ijk=1,2
                    qresult(1)=qqn(1)
                    qresult(2)=qqn(2)
                    qresult(3)=qqn(3)
                    qresult(4)=qqn(4)
                    iq4neg=ijk
                    if(ijk.eq.2) qresult(4)=-1.*qqn(4)
!  take the inverse rotation, i.e. apply the switching symmetry

                    if(qresult(1).ge.0.0) then
                       if(qresult(2).ge.0.0) then
                          if(qresult(3).ge.0.0) then
!   these IFs ensure that 0<=q1, 0<=q2, 0<=q3 to place it in the fundamental zone
!  which is a cube of size=1., between 0 and 1 in all 3 axes
                             if(qresult(4).gt.qmax) then
!  so now we test to see if we have a smaller angle, which closes the box
                                qmax=qresult(4)
                                index1=i
                                index2=j
!			        index3=0
                                index1a=0
                                if(ijk.eq.2) index1a=1
                                index2a=0
                                if(kjl.eq.2) index2a=1
!  record the result if we have found a quat in the fund. zone, and a smaller angle
                             endif
                          endif
                       endif
                    endif
 210             continue
 185          continue
 190       continue
 200    continue

	if(index1.eq.0.or.index2.eq.0) stop 'DISQUAT failed!'

        call presymm(qq,index1,quint)

        call postsymm(quint,index2,qqn)
!		 write(*,"('DISQUAT: qqn = ',4f8.3)") qqn
        do 280, k=1,4
           if(index2a.eq.1) then
!  if we had to take the negative of the Q, then do so
!  to the answer

              qresult(k) = -1.0 * qqn(k)
!  can take negative of Q because identity = +/-(0,0,0,1)
           else
              qresult(k) = qqn(k)
           endif
 280    continue

	if(index1a.eq.1) then
           qresult(4) = -1. * qresult(4)
!  restore the 4th component if disorient found for ijk=1
	endif

!	write(*,*) 'DISQUAT: qmax= ',qmax

        if (qmax .gt. 1. ) qmax = 1.
	thetamin = 360. / pi * acos(qmax)
!  11 xii 10 - this subr now has PI defined
	return
	end

!
!  ____________________________
!
!
	subroutine q2rod(d,quat)
	real d(3),quat(4),rtmp,rnorm,rnorm2
!  converts a quaternion to a Rodrigues vector

	rnorm=sqrt(quat(1)**2+quat(2)**2+quat(3)**2+quat(4)**2)
	if(rnorm.lt.1e-5) stop 'Q2ROD: too small a quaternion!'
	if(abs(rnorm-1.0).gt.1.e-3) then
	   write(*,*) 'Q2ROD: quat required normalization'
	   do i=1,4
	      quat(i)=quat(i)/rnorm
	   enddo
	endif

	if(quat(4).gt.1.e-2) then
	   d(1)=quat(1)/quat(4)
	   d(2)=quat(2)/quat(4)
	   d(3)=quat(3)/quat(4)
	else
!  we calculate sine(theta/2) from quat1-3
!  and scale by tangent/sine in order to make
!  the system deal with large numbers!
!
	   rtmp=sqrt(quat(1)**2+quat(2)**2+quat(3)**2)
!  this will be close to unity
	   stheta=asin(rtmp)
	   ttheta=tan(stheta)	! and this will be large
	   d(1)=quat(1)*ttheta/stheta
	   d(2)=quat(2)*ttheta/stheta
	   d(3)=quat(3)*ttheta/stheta
	endif

	return
	end

!
!  -------------------
!
      subroutine euler(a,iopt,nomen,d1,d2,d3,ior,kerr)
!                          Last revision 20nov90 UFK
!      common a(3,3),grvol(1152),epsga(5),ist1,ist2,sqr3,sqrh,ident(3,3)
!  SPECIAL VERSION WITHOUT COMMON BLOCK
      real a(3,3),d1,d2,d3,th,sth,cth,sph,cph,sps,cps,ps,ph,dth,dph,dps
      character nomen
      save kor
      real pi
      parameter ( pi = 3.14159265 )
      real rad
      parameter ( rad = 57.29578 )

! CODE::

! *** this subroutine calculates the euler angles associated with the
!     transformation matrix a(i,j) if iopt=1 and viceversa if iopt=2
! ***  Note that a is sample (rows) in terms of crystal (columns);
!      -- opposite of standard g (e.g.Bunge) - this is Canova's
! ***  Note that in this version, the Euler angles are defined symmetrically:
!      so that interchanging phi and psi means transposing a.
!      ("Kocks" nomen: defined going from +X to +Y in both COD and SOD)
! ***  However, other angle conventions are translated, according to
!  nomen="K" - Kocks (as internally) -- also sometimes "N"...
!        "R" - Roe          (Psi=psi, Phi=180-phi)
!        "B" - Bunge        (phi1=90+psi, Phi=Theta, phi2=90-phi)
!        any other - Canova (Theta first, phiC=90+phi, omega=90-psi)
! ***   Note: only in symm. notation does a point with all Euler angles
!             between 0 and 90 deg appear in the +x,+y quadrant!
!       If you want to see an individual point in this quadrant and are:
!       using Roe,    the third angle must be between 90 and 180;
!             Bunge,      first                                 ;
!             Canova,     second                                .
! ***  Input and output Euler angles d1,d2,d3 in degrees
!
      goto(5,20),iopt
    5 if(abs(a(3,3)) .ge. 0.999) goto 10
      th=acos(a(3,3))
      sth=sin(th)
!
	if((abs(a(2,3)/sth).lt.1e-35).and.
     &      (abs(a(1,3)/sth).lt.1e-35)) then
		ps=pi/4.
	else
      ps=atan2(a(2,3)/sth,a(1,3)/sth)
	endif
!
	if((abs(a(3,2)/sth).lt.1e-35).and.
     &      (abs(a(3,1)/sth).lt.1e-35)) then
		ph=pi/4.
	else
      ph=atan2(a(3,2)/sth,a(3,1)/sth)
	endif
!cccc   if it bombs out here, both a-components in one arg. are zero
!       (this should not be possible, but has happened, probably fixed)
!  ADR: i 01 - above is the fix!!
!
      go to 15
!
   10 	if((abs(a(1,2)/sth).lt.1e-35).and.
     &      (abs(a(1,1)/sth).lt.1e-35)) then
		ps=pi/4.
	else
		ps=0.5*atan2(a(1,2),-a(1,1))
	endif
!
      ph=-ps
!       The above still have the problem that they give too many
!       equivalents of the same grain in DIOROUT and density file. Therefore:
      if(kerr.eq.1.and.kor.ne.ior) then
      print*,'NOTE: grain',ior,' has Theta < 1 deg.: sometimes problems'
        print*
        kor=ior
      endif
!
   15 dth=th*rad
      dph=ph*rad
      dps=ps*rad
      d1=dps
      d2=dth
      if(nomen.eq.'K'.or.nomen.eq.'N') then
        d3=dph
      elseif(nomen.eq.'R') then
        d3=180.-dph
      elseif(nomen.eq.'B') then
        d1=dps+90.
        d3=90.-dph
      else
        d1=dth
        d2=dph+90.
        d3=90.-dps
      endif
      if(d1.ge.360.) d1=d1-360.
      if(d3.ge.360.) d3=d3-360.
      if(d1.lt.0.) d1=d1+360.
      if(d3.lt.0.) d3=d3+360.
      return
!  *************************************
   20 dth=d2
      dps=d1
      if(nomen.eq.'K') then
        dph=d3
      elseif(nomen.eq.'R') then
        dph=180.-d3
      elseif(nomen.eq.'B') then
        dps=d1-90.
        dph=90.-d3
      else
        dth=d1
        dph=d2-90.
        dps=90.-d3
      endif
      ph=dph/rad
      th=dth/rad
      ps=dps/rad
      sph=sin(ph)
      cph=cos(ph)
      sth=sin(th)
      cth=cos(th)
      sps=sin(ps)
      cps=cos(ps)
      a(1,1)=-sps*sph-cph*cps*cth
      a(2,1)=cps*sph-cph*sps*cth
      a(3,1)=cph*sth
      a(1,2)=sps*cph-sph*cps*cth
      a(2,2)=-cph*cps-sph*sps*cth
      a(3,2)=sth*sph
      a(1,3)=sth*cps
      a(2,3)=sps*sth
      a(3,3)=cth
      return
      end

	
